using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using WebseitenMonitor.DataAccess;
using System.Web.Security;

namespace WebseitenMonitor.Admin
{
	/// <summary>
	/// Summary description for UserManagement_Main.
	/// </summary>
	public class UserManagement_Main : System.Web.UI.Page
	{
		protected System.Web.UI.WebControls.DropDownList UserList;
		protected System.Web.UI.WebControls.Button NewUserButton;
		protected System.Web.UI.WebControls.Button EditUserButton;
		protected System.Web.UI.WebControls.Button LogoutButton;
		protected System.Web.UI.WebControls.DropDownList AdminList;
		protected System.Web.UI.WebControls.Button NewAdminButton;
		protected System.Web.UI.WebControls.Button EditAdminButton;
		protected System.Web.UI.WebControls.Button DeleteAdminButton;
		protected System.Web.UI.WebControls.Button DeleteUserButton;
		protected System.Web.UI.WebControls.Label ErrorLabel;
	
		private void Page_Load(object sender, System.EventArgs e)
		{
			Hashtable users;
			string errorMsg;
			UsersMediator.ReadResult readRes = UsersMediator.LoadUsers(new string[] {"users"}, out users, out errorMsg);
			if(readRes == UsersMediator.ReadResult.Ok) 
			{
				this.UserList.DataSource = users;
				this.UserList.DataTextField = "Value";
				this.UserList.DataValueField = "Key";
				this.UserList.DataBind();
			}
			Hashtable admins;
			readRes = UsersMediator.LoadUsers(new string[] {"admins"}, out admins, out errorMsg);
			if(readRes == UsersMediator.ReadResult.Ok)
			{
				this.AdminList.DataSource = admins;
				this.AdminList.DataTextField = "Value";
				this.AdminList.DataValueField = "Key";
				this.AdminList.DataBind();
			}
			if(admins.Count <= 1)
				this.DeleteAdminButton.Enabled = false;
		}

		protected void NewUserButtonClicked(object sender, EventArgs args) 
		{
			Session["NewUser_Role"] = "users";
			Response.Redirect("./UserManagement_AddUser.aspx");
		}

		protected void EditUserButtonClicked(object sender, EventArgs args) 
		{
			ErrorLabel.Text = "";
			if(UserList.SelectedItem != null) 
			{
				Session["EditUser_Id"] = int.Parse(UserList.SelectedItem.Value);
				Session["EditUser_Name"] = UserList.SelectedItem.Text;
				Response.Redirect("./UserManagement_EditUser.aspx");
			} 
			else 
			{
				ErrorLabel.Text = "Es wurde kein Element selektiert!";
			}
		}

		protected void DeleteUserButtonClicked(object sender, EventArgs args) 
		{
			ErrorLabel.Text = "";
			if(UserList.SelectedItem != null) 
			{
				int userId = int.Parse(UserList.SelectedItem.Value);
				string errorMsg;
				UsersMediator.DeleteResult res = UsersMediator.DeleteUser(userId, out errorMsg);
				if(res != UsersMediator.DeleteResult.Ok) 
				{
					ErrorLabel.Text = errorMsg;
				} else
					Response.Redirect("./UserManagement_Main.aspx");
			} 
			else 
			{
				ErrorLabel.Text = "Es wurde kein Element selektiert!";
			}
		}

		protected void NewAdminButtonClicked(object sender, EventArgs args) 
		{
			Session["NewUser_Role"] = "admins";
			Response.Redirect("./UserManagement_AddUser.aspx");
		}

		protected void EditAdminButtonClicked(object sender, EventArgs args) 
		{
			ErrorLabel.Text = "";
			if(AdminList.SelectedItem != null) 
			{
				Session["EditUser_Id"] = int.Parse(AdminList.SelectedItem.Value);
				Session["EditUser_Name"] = AdminList.SelectedItem.Text;
				Response.Redirect("./UserManagement_EditUser.aspx");
			} 
			else 
			{
				ErrorLabel.Text = "Es wurde kein Element selektiert!";
			}
		}

		protected void DeleteAdminButtonClicked(object sender, EventArgs args) 
		{
			ErrorLabel.Text = "";
			if(AdminList.SelectedItem != null) 
			{
				int userId = int.Parse(AdminList.SelectedItem.Value);
				string errorMsg;
				UsersMediator.DeleteResult res = UsersMediator.DeleteUser(userId, out errorMsg);
				if(res != UsersMediator.DeleteResult.Ok) 
				{
					ErrorLabel.Text = errorMsg;
				} 
				else
					Response.Redirect("./UserManagement_Main.aspx");
			} 
			else 
			{
				ErrorLabel.Text = "Es wurde kein Element selektiert!";
			}
		}

		protected void LogoutClicked(object sender, EventArgs args) 
		{
			FormsAuthentication.SignOut();
			Session.Clear();
			this.Response.Redirect("../Login.aspx");
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion
	}
}
