using System;

namespace WebseitenMonitor.DomainObjects.Behaviour 
{	
	public enum ChangeStateEnum {Unchanged = 0, Changed = 1, Error = 2, Init = 3, New = 4}
	public enum ObservationKind {ChangeObservation = 0, UserDefinedMessageObservation = 1 }
	
	public enum CheckResult {Ok, Error, InvalidParams}
	
	/// <summary>
	/// This class describes the things in common to all Observations.
	/// It corresponds to a central domain object in the WebseitenMonitor application, as it represents the
	/// things in common for each resource to monitor.
	/// </summary>
	/// <author>Stefan Preuer, Manuel Wallnfer</author>
	public abstract class Observation 
	{
		internal static readonly DateTime InitDateTime = new DateTime();	// Used as init-Date (e.g.: corresponds to a DBNull in the database)
		// Note: DateTime is struct - null not possible
		protected int id;						// Unique id of the observation
		protected string name;					// User defined name of the observation
		protected Uri url;						// Url of the resourse to observe
		protected DateTime date;				// DateTime the changeState belongs to (last observation time)
		protected int checkCounterTimeStamp;	// Global integer timestamp of observation - to compare relevance to the present
												// against other observations.
		protected ChangeStateEnum changeState;	// The summarized change of the obseravation in comparison to the state before
												// the current observation check.

		internal Observation (
			int id,
			string name,
			Uri url,
			DateTime date,
			int checkCounterTimeStamp,
			ChangeStateEnum changeState) 
		{
			this.id = id;
			this.name = name;
			this.url = url;
			this.date = date;
			this.checkCounterTimeStamp = checkCounterTimeStamp;
			this.changeState = changeState;
		}

		// Getter Properties for Observation fields

		internal int Id 
		{
			get { return this.id; }
		}
		
		internal string Name 
		{
			get { return this.name; }
		}

		internal Uri Url 
		{
			get { return this.url; }
		}

		internal DateTime Date 
		{
			get { return this.date; }
		}
		
		internal int CheckCounterTimeStamp 
		{
			get { return this.checkCounterTimeStamp; }
		}

		internal ChangeStateEnum ChangeState 
		{
			get { return this.changeState; }
		}

		// This abstract method represents the method to check the Observation for changes and must
		// be overridden/implemented by each concrete Observation
		internal abstract CheckResult Check(int newCheckCounterTimeStamp, out string errorMsg);

	} // end: abstract class Observation
}