using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using log4net;
using System.Web.Security;
using WebseitenMonitor.Security;
using System.Configuration;
using WebseitenMonitor.DataAccess;

namespace WebseitenMonitor
{
	/// <summary>
	/// Summary description for Login.
	/// </summary>
	public class Login : System.Web.UI.Page
	{
		private static readonly ILog log = LogManager.GetLogger(typeof(Login));

		protected System.Web.UI.WebControls.TextBox UsernameBox;
		protected System.Web.UI.WebControls.TextBox PasswordBox;
		protected System.Web.UI.WebControls.Button LoginButton;
		protected System.Web.UI.WebControls.Label Output;
	
		private void Page_Load(object sender, System.EventArgs e)
		{
			// Put user code to initialize the page here
		}
		
		/// <summary>
		/// Authenticate User by the credentials typed in at Login.aspx (Username and Password).
		/// If successful - redirect to corresponding entry-url.
		/// Else - show error-message in Output-Label at Login.aspx.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		protected void OnLogIn(object sender, System.EventArgs e) 
		{
			log.Debug("Login request with username: \"" + UsernameBox.Text + "\".");
			// Validate login-credential for syntactical correctness
			string errorMsg;
			if(!this.validateLoginCredentials(UsernameBox.Text, PasswordBox.Text, out errorMsg)) 
			{
				log.Info("Validation of login credentials failed (Username: " + UsernameBox.Text + ").");
				Output.Text = "Ung&uuml;ltige Eingabe: <br/>" + errorMsg;
			} 
			else 
			{
				// assertion: login credential are syntactical valid
				// Authenticate user
				UserDataWrapper userData;
				AuthenticationResult authRes = AuthenticationProvider.authenticate(
					UsernameBox.Text,
					PasswordBox.Text,
					out userData);

				if(authRes == AuthenticationResult.Positive)
				{
					Session.Clear();
					if(userData == null) 
					{
						log.Fatal("Login failed due to internal error. Returned UserDataWrapper-object is null.");
						Output.Text = "Interner Fehler aufgetreten!";
						return;
					}
					// Set users CheckCounter into session-state
					int checkCounter;
					UsersMediator.ReadResult res = UsersMediator.ReadUsersCheckCounter(userData.Userid, out checkCounter, out errorMsg);
					if(res == UsersMediator.ReadResult.Ok)
						Session["UserCheckCounter"] = checkCounter;
					else 
					{
						log.Error("Login failed because read of users CheckCounter failed!");
						Output.Text = "Fehler beim Datenbankzugriff aufgetreten!";
						return;
					}

					// assertion: Authentication was successfull, userData holds user-corresponding wrapped data.
					// check whether user is in one of the expected roles to go on
					if(!userData.IsUserInRole("users") && ! userData.IsUserInRole("admins")) 
					{
						log.Warn("Successfully authenticated user (Username: " + UsernameBox.Text + ") not in users or admins role. Login denied!");
						Output.Text = "Sie haben keine Loginberechtigung!";
						return;
					}
					// assertion: user successfully authenticated
					// set non-persistent authentication cookie
					FormsAuthentication.SetAuthCookie(userData.CoreString, false);
					// Redirect user corresponding to role
					if(userData.IsUserInRole("admins")) 
					{
						log.Info("User (Username: " + UsernameBox.Text + ", Role: admins) successfully authenticated, logged in and AuthCookie set.");
						Response.Redirect(ConfigurationSettings.AppSettings["LoginRedirectPage_admins"]);
					}
					else if(userData.IsUserInRole("users")) 
					{
						log.Info("User (Username: " + UsernameBox.Text + ", Role: users) successfully authenticated, logged in and AuthCookie set.");
						Response.Redirect(ConfigurationSettings.AppSettings["LoginRedirectPage_users"]);
					}
				} 
				else if(authRes == AuthenticationResult.Negative)
				{
					// assertion: user authentication failed - wrong login credentials
					log.Warn("Authentication was negative for user: \"" + UsernameBox.Text + "\".");
					Output.Text = "Benutzername oder Passwort ung&uuml;ltig!";
				} 
				else // authRes == AuthenticationResult.Error 
				{
					log.Error("Authentication failed (error) for user: \"" + UsernameBox.Text + "\".");
					Output.Text = "Technisches Problem whrend Authentifizierung aufgetreten!";
				}
			}
		}
		
		/// <summary>
		/// This method validates the login-credentials username and password for syntacitcal correctness.
		/// It should prevent spoofing-attacks, because username and password are queried against a database.
		/// </summary>
		/// <param name="username">Username to validate.</param>
		/// <param name="password">Password to validate.</param>
		/// <param name="errorMsg">User visible error Message if validation failes (false returned).</param>
		/// <returns>true .. ok, false .. fail</returns>	
		private bool validateLoginCredentials(string username, string password, out string errorMsg) 
		{
			if(username == null || password == null) // should actually never happen 
				errorMsg = "Eingabe konnte nicht verarbeitet werden!";
			else if(username.Length == 0) 
				errorMsg = "Es wurde kein Benutzername eingegeben!";
			else if(password.Length == 0)
				errorMsg = "Es wurde kein Passwort eingegeben!";
			else 
			{
				errorMsg = "";
				return true;
			}
			return false;
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.ID = "LoginForm";
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion
	}
}
