using System;
using System.Text;
using System.Data;
using System.Data.SqlClient;
using System.Web.Security;
using System.Configuration;
using log4net;

namespace WebseitenMonitor.Security 
{
	public class PasswordUtils 
	{
		internal static string GeneratePasswordHashString(string pwd) 
		{
			return FormsAuthentication.HashPasswordForStoringInConfigFile(pwd, "MD5");
		}
	}

	/// <summary>
	/// This class represents a Wrapper for Userinformation (authenticated?, userid, username, password and roles) and allows it
	/// to represent this information as one single string.
	/// It is necessary to encode this detailed userinformation in one single usernamestring from which the ASP.NET
	/// framework can generate an Authentication-Cookie (<c>System.Web.Security.FormsAuthentication.SetAuthCookie(string username,bool persistent)</c>).
	/// So it's possible to obtain detailed userinformation from the Authentication-Cookie, which is necessary if no Session-object is accessible
	/// (e.g.: in the Application_AuthenticateRequest handler method in Global.asax)
	/// </summary>
	/// <author>Stefan Preuer, 0055832</author>
	/// <creationDate>28.12.2004</creationDate>
	public class UserDataWrapper 
	{	
		/// <summary>
		/// Generates a core-string which can be wrapped by an UserDataWrapper-object.
		/// </summary>
		/// <param name="authenticated">true - user is authenticated, false - user is anonymous</param>
		/// <param name="username">May be null.</param>
		/// <param name="password">May be null.</param>
		/// <param name="roles">roles and the elements of roles may be null.</param>
		/// <returns>core-string</returns>
		public static string GenerateCoreString(bool authenticated, int userid, string loginname, string password, string[] roles) 
		{	

			// Syntax: <0 .. not authenticated | 1 .. authenticated><userid-length>:userid<loginname-length>:loginname<password-length>:password<roles-array-length>:<roles-elem-0-lenght>:roles[0]<roles-elem-1-length>:roles[1] ...
			//   Note: A length of -1 represents null
			StringBuilder tmp = new StringBuilder();
			if(authenticated)
				tmp.Append("1");
			else
				tmp.Append("0");
			string useridString = userid.ToString();
			if(userid >= 0) {tmp.Append(useridString.Length + ":" + useridString);}
			else {tmp.Append(-1 + ":");}
			if(loginname != null) {tmp.Append(loginname.Length + ":" + loginname);}
			else {tmp.Append(-1 + ":");}
			if(password != null) {tmp.Append(password.Length + ":" + password);}
			else {tmp.Append(-1 + ":");}
			if(roles != null)
			{
				tmp.Append(roles.Length + ":");
				for(int i=0; i<roles.Length; i++) 
				{
					if(roles[i] != null) 
					{
						tmp.Append(roles[i].Length + ":");
						tmp.Append(roles[i]);
					}
					else 
					{
						tmp.Append(-1 + ":");
					}
				}
			}
			else {tmp.Append(-1 + ":");}

			return tmp.ToString();
		} // end: method GenerateCoreString
		
		private bool authenticated;
		private string useridString;
		private string loginname;
		private string password;
		private string[] roles;
		
		/// <summary>
		/// Generates a Wrapper-object for a core-string generated by <c>UserDataWrapper.generateCoreString()</c>.
		/// Note: Only call this method with a core-string generated by <c>UserDataWrapper.generateCoreString()</c>, 
		///       because the method is not tested to be stable for syntactical incorrect core-strings.
		/// </summary>
		/// <param name="coreString">The core-string to be wrapped.</param>
		public UserDataWrapper(string coreString) 
		{
			if(coreString == null) 
			{
				this.authenticated = false;
				this.useridString = null;
				this.loginname = null;
				this.password = null;
				this.roles = null;
			} 
			else 
			{
				if(coreString.Length >= 1) 
				{
					if(coreString[0] == '0')
						this.authenticated = false;
					else
						this.authenticated = true;
				}
				int i = 1, j = 1;
				int length;
				bool stop = false;
				// parse userid
				while(j < coreString.Length && !stop) 
				{
					if(coreString[j] == ':')
						stop = true;
					j++;
				}
				length = Convert.ToInt32(coreString.Substring(i,j-i-1));
				useridString = (length <= -1)?null:coreString.Substring(j,length);
				if(length > -1)
					i = j = j + length;
				else
					i = j;
				stop = false;
				// parse username
				while(j < coreString.Length && !stop) 
				{
					if(coreString[j] == ':')
						stop = true;
					j++;
				}
				length = Convert.ToInt32(coreString.Substring(i,j-i-1));
				loginname = (length <= -1)?null:coreString.Substring(j,length);
				if(length > -1)
					i = j = j + length;
				else
					i = j;
				stop = false;
				// parse password
				while(j < coreString.Length && !stop) 
				{
					if(coreString[j] == ':')
						stop = true;
					j++;
				}
				length = Convert.ToInt32(coreString.Substring(i,j-i-1));
				password = (length <= -1)?null:coreString.Substring(j,length);
				if(length > -1)
					i = j = j + length;
				else
					i = j;
				stop = false;
				// parse roles
				// parse password
				//   parse roles - arraylength
				while(j < coreString.Length && !stop) 
				{
					if(coreString[j] == ':')
						stop = true;
					j++;
				}
				length = Convert.ToInt32(coreString.Substring(i,j-i-1));
				i = j; stop = false;
				if(length > -1) 
				{
					int numRoles = length;
					roles = new string[length];
					for(int k = 0; k < numRoles; k++) 
					{
						while(j < coreString.Length && !stop) 
						{
							if(coreString[j] == ':')
								stop = true;
							j++;
						}
						length = Convert.ToInt32(coreString.Substring(i,j-i-1));
						roles[k] = (length <= -1)?null:coreString.Substring(j,length);
						if(length > -1)
							i = j = j + length;
						else
							i = j;
						stop = false;
					}
				}
				else 
				{
					roles = null;
				}
			}
		} // end: constructor UserDataWrapper
		
		/// <summary>
		/// Authenticated-component
		/// </summary>
		public bool Authenticated 
		{
			get { return this.authenticated; }
		}
		
		/// <summary>
		/// Userid-component
		/// </summary>
		public int Userid 
		{
			get 
			{ 
				if(this.useridString == null)
					return -1;
				else
					return Convert.ToInt32(this.useridString);
			}
		}

		/// <summary>
		/// Loginname-component.
		/// </summary>
		public string Loginname 
		{
			get { return this.loginname; }
		}
		
		/// <summary>
		/// Password-component.
		/// </summary>
		public string Password 
		{
			get { return this.password; }
		}
		
		/// <summary>
		/// Roles-component.
		/// </summary>
		public string[] Roles 
		{
			get { return this.roles; }
		}

		public string CoreString 
		{
			get { return GenerateCoreString(this.authenticated, this.Userid, this.loginname, this.password, this.roles); }
		}

		public bool IsUserInRole(string role) 
		{
			if(this.Roles == null)
				return false;
			foreach(string tmp in this.Roles)
				if(role.Equals(tmp))
					return true;
			return false;
		}
	} // end: class UserDataWrapper
	
	internal enum AuthenticationResult {Positive, Negative, Error};

	/// <summary>
	/// This class provides support for authentication against the WEBSEITENMONITOR-Database.
	/// </summary>
	internal class AuthenticationProvider 
	{
		private static readonly ILog log = LogManager.GetLogger(typeof(AuthenticationProvider));
		private static readonly string DbTableOwnerPrefix = ((string)ConfigurationSettings.AppSettings["DbTableOwnerPrefix"]).Trim();
		
		/// <summary>
		/// This method tries to authenticate against the WEBSEITENMONITOR-database by the passed credentials (username, password).
		/// The wrapper-object returned by the outputparameter userData holds informations about the authenticated users necessary 
		/// for the application.
		/// Very important is the Userid from the databasetable, because it identifies a user uniquely.
		/// The Roles are also important for Authorization reasons.
		/// Note: Only the Authenticated, Userid, Loginname and Roles components are set accordingly.
		/// </summary>
		/// <param name="username">Username part of authentication-credentials</param>
		/// <param name="password">Password part of authentication-credentials</param>
		/// <param name="userData">Wrapper object for important data of authenticated user.</param>
		/// <returns>Positive .. authentic, Negative .. username or password invalid, Error .. an error occured</returns>
		internal static AuthenticationResult authenticate(string username, string password, out UserDataWrapper userData) 
		{
			int DbId = -1;
			string DbRole = null;
			string DbPwd = null;
			string conStr = ConfigurationSettings.AppSettings["AuthenticationDbConnectionString"];
			IDbConnection con = null;
			try 
			{
				con = new SqlConnection(conStr);
				con.Open();
				IDbCommand cmd = con.CreateCommand();
				cmd.CommandText = "SELECT * FROM " + DbTableOwnerPrefix + "Users WHERE LoginName = '" + username + "'";
				IDataReader reader = cmd.ExecuteReader();
				int resCount = 0;
				while(reader.Read()) 
				{
					resCount++;
					if(resCount > 1)
						break;
					DbId = (int)reader["ID"];
					DbPwd = (string)reader["PwdHash"];
					DbRole = (string)reader["Role"];
				}
				reader.Close();
				if(resCount > 1) // should never occure - application-consistence selftest
				{
					// assertion: there are multible users with same LoginName - should never occure
					log.Fatal("LoginName: " + username + " not unique in Users-databasetable!");
					userData = null;
					return AuthenticationResult.Error;
				}
				if(resCount == 0) // no such user in DB
				{
					userData = null;
					return AuthenticationResult.Negative;
				}
				if(DbId < 0 && resCount == 1) // should never occure - application-consistence selftest
				{
					log.Fatal("LoginName: " + username + " has negative id in Users-databasetable!");
					userData = null;
					return AuthenticationResult.Error;
				}
				// Check Password
				if(PasswordUtils.GeneratePasswordHashString(password).Equals(DbPwd))
				{
					// assertion: password is correct (corresponds to the hash in database)
					string[] roleArr = (DbRole == null)?null:new string[] {DbRole};
					userData = new UserDataWrapper(UserDataWrapper.GenerateCoreString(
						true,
						DbId,
						username,
						null,
						roleArr));
					return AuthenticationResult.Positive;
					} 
				else 
				{
					// assertion: password is incorrect (doesn't correspond to hash in database)
					userData = null;
					return AuthenticationResult.Negative;
				}
			} 
			catch(Exception e)
			{
				log.Error("Exception occured during authentication against database", e);
				userData = null;
				return AuthenticationResult.Error;
			} 
			finally 
			{
				try { if(con != null) con.Close(); } 
				catch(Exception ex) 
				{
					log.Warn("Unable to close connection - exception occured.", ex);
				}
			}
		} // end: method authenticate
	} // end: class AuthenticationProvider
}