namespace WebseitenMonitor.Tests 
{
	using System;
	using NUnit.Framework;
	using WebseitenMonitor.Security;

	[TestFixture]
	public class UserDataWrapperTests
	{	
		[Test]
		public void testpositiveGenerateCoreString() 
		{
			string expected = "0-1:-1:-1:-1:";
			Assert.AreEqual(expected,UserDataWrapper.GenerateCoreString(false,-1,null,null,null));
			
			expected = "0-1:0:-1:-1:";
			Assert.AreEqual(expected,UserDataWrapper.GenerateCoreString(false,-1,"",null,null));
			
			expected = "11:55:Maier5:Franz2:5:Admin4:User";
			Assert.AreEqual(expected,UserDataWrapper.GenerateCoreString(true,5,"Maier","Franz",new string[] {"Admin","User"}));
			
			expected = "12:105:Maier5:Franz0:";
			Assert.AreEqual(expected,UserDataWrapper.GenerateCoreString(true,10,"Maier","Franz",new string[0]));
			
			expected = "12:104:Hans5:Maier4:-1:0:1:a2:ab";
			string[] roles = new string[4];
			roles[0] = null; roles[1] = ""; roles[2] = "a"; roles[3] = "ab";
			Assert.AreEqual(expected,UserDataWrapper.GenerateCoreString(true,10,"Hans","Maier",roles));
		}

		[Test]
		public void testpositiveParse() 
		{	
			
			UserDataWrapper udw = new UserDataWrapper("0-1:-1:-1:-1:");
			Assert.IsFalse(udw.Authenticated);
			Assert.AreEqual(-1,udw.Userid);
			Assert.IsNull(udw.Loginname);
			Assert.IsNull(udw.Password);
			Assert.IsNull(udw.Roles);
			
			udw = new UserDataWrapper("01:00:-1:-1:");
			Assert.IsFalse(udw.Authenticated);
			Assert.AreEqual(0,udw.Userid);
			Assert.AreEqual("", udw.Loginname);
			Assert.IsNull(udw.Password);
			Assert.IsNull(udw.Roles);
			
			udw = new UserDataWrapper("12:104:Hans5:Maier2:5:Admin4:User");
			Assert.IsTrue(udw.Authenticated);
			Assert.AreEqual(10,udw.Userid);
			Assert.AreEqual("Hans",udw.Loginname);
			Assert.AreEqual("Maier",udw.Password);
			Assert.AreEqual(2,udw.Roles.Length);
			Assert.AreEqual("Admin",udw.Roles[0]);
			Assert.AreEqual("User",udw.Roles[1]);
			
			udw = new UserDataWrapper("12:104:Hans5:Maier0:");
			Assert.IsTrue(udw.Authenticated);
			Assert.AreEqual(0,udw.Roles.Length);

			udw = new UserDataWrapper("12:104:Hans5:Maier4:-1:0:1:a2:ab");
			Assert.IsTrue(udw.Authenticated);
			Assert.AreEqual(4,udw.Roles.Length);
			Assert.AreEqual(null,udw.Roles[0]);
			Assert.AreEqual("",udw.Roles[1]);
			Assert.AreEqual("a",udw.Roles[2]);
			Assert.AreEqual("ab",udw.Roles[3]);
		}
	}
}