namespace WebseitenMonitor.Tests
{
	using System;
	using NUnit.Framework;
	using WebseitenMonitor.DomainObjects.Behaviour;
	using System.Collections;
	using System.IO;
	using System.Text;

	[TestFixture]
	public class BasicTests 
	{	
		private string errorMessage;

		[Test] public void CheckInvalidSearchPattern() 
		{	
			Assert.IsFalse(UserDefinedMessageObservation.CheckSearchPattern("ab[cd",out errorMessage));
			//Console.WriteLine(errorMessage);
		}

		[Test] public void CheckValidMessageTemplate01() 
		{
			Assert.IsTrue(UserDefinedMessageObservation.CheckMessageTemplate("$0 ab $09a$0}$10$$${-_+azAZ}", out errorMessage));
		}
		[Test] public void CheckValidMessageTemplate02() 
		{
			Assert.IsTrue(UserDefinedMessageObservation.CheckMessageTemplate("$$", out errorMessage));
		}
		[Test] public void CheckValidMessageTemplate03() 
		{
			Assert.IsTrue(UserDefinedMessageObservation.CheckMessageTemplate("${}", out errorMessage));
		}
		
		[Test] public void CheckInvalidMessageTemplate01() 
		{
			Assert.IsFalse(UserDefinedMessageObservation.CheckMessageTemplate("$", out errorMessage));
		}
		[Test] public void CheckInvalidMessageTemplate02() 
		{
			Assert.IsFalse(UserDefinedMessageObservation.CheckMessageTemplate("$0 abc ${abc def ", out errorMessage));
		}
		[Test] public void CheckInvalidMessageTemplate03() 
		{
			Assert.IsFalse(UserDefinedMessageObservation.CheckMessageTemplate("${ab<}", out errorMessage));
		}
		[Test] public void CheckInvalidMessageTemplate04() 
		{
			Assert.IsFalse(UserDefinedMessageObservation.CheckMessageTemplate("$a", out errorMessage));
		}
		[Test] public void CheckInvalidMessageTemplate05() 
		{
			Assert.IsFalse(UserDefinedMessageObservation.CheckMessageTemplate("${&}", out errorMessage));
		}
	}
	
	/*
	[TestFixture]
	public class UpdateSearchPatternParameterDictionaryTests 
	{	
		private UserDefinedMessageObservation behaviour;

		private string text1 = "Dies ist ein einfacher Testtext.";
		
		[SetUp] public void init() 
		{
			this.behaviour = new UserDefinedMessageObservation();
		}

		[Test] public void SimpleTest1() 
		{
			string errorMessage;
			IDictionary dictionary;
			this.behaviour.SetSearchPattern("e(i)n(?<param1>f)(y*)",out errorMessage);
			UserDefinedMessageObservation.UpdateSearchPatternParameterResult res = this.behaviour.updateSearchPatternParameterDictionary(this.text1, out dictionary, out errorMessage);
			Console.WriteLine(errorMessage);
			Assert.AreEqual(UserDefinedMessageObservation.UpdateSearchPatternParameterResult.Match, res);
			Assert.AreEqual(dictionary["0"],"einf");
			Assert.AreEqual(dictionary["1"],"i");
			Assert.AreEqual(dictionary["param1"],"f");
			Assert.AreEqual(dictionary["2"],"");
			Assert.IsNull(dictionary["3"]);
		}
	}*/

	[TestFixture]
	public class MessageTemplateScannerTests 
	{
		[Test] public void Test1() 
		{
			StringReader sr = new StringReader("ab$034${ab} ${ ab $12 $$$-4");
			UserDefinedMessageObservation.MessageTemplateScanner scanner = new UserDefinedMessageObservation.MessageTemplateScanner();
			scanner.Init(sr);
			UserDefinedMessageObservation.MessageTemplateParameterToken t = scanner.NextParameter();
			Assert.AreEqual(UserDefinedMessageObservation.MessageTemplateParameterToken.numbered, t.kind);
			Assert.AreEqual("034",t.name);
			Assert.AreEqual(34,t.number);
			Assert.AreEqual(2,t.pos);
			Assert.AreEqual(4,t.length);
			t = scanner.NextParameter();
			Assert.AreEqual(UserDefinedMessageObservation.MessageTemplateParameterToken.named, t.kind);
			Assert.AreEqual("ab",t.name);
			Assert.AreEqual(0,t.number);
			Assert.AreEqual(6,t.pos);
			Assert.AreEqual(5,t.length);
			t = scanner.NextParameter();
			Assert.AreEqual(UserDefinedMessageObservation.MessageTemplateParameterToken.numbered, t.kind);
			Assert.AreEqual("12",t.name);
			Assert.AreEqual(12,t.number);
			Assert.AreEqual(18,t.pos);
			Assert.AreEqual(3,t.length);
			t = scanner.NextParameter();
			Assert.AreEqual(UserDefinedMessageObservation.MessageTemplateParameterToken.eof, t.kind);
			Assert.AreEqual(null,t.name);
			Assert.AreEqual(0,t.number);
			Assert.AreEqual(27,t.pos);
			Assert.AreEqual(0,t.length);
		}
	}
}