namespace WebseitenMonitor.Tests
{
	using System;
	using NUnit.Framework;
	using WebseitenMonitor.DomainObjects.Behaviour;
	using System.Collections;
	using System.IO;
	using System.Text;
	using mshtml;
	
	[TestFixture]
	public class ClientSelectedElementsWrapperTests 
	{
		[Test] public void SimpleTest() 
		{
			string sepd = "0;2;p,2,3,25,3,36;i,0,2,29,5,34;>1;1;h1,0,2,50,1,60;>";
			UserDefinedMessageObservation.ClientSelectedElementsWrapper w = new UserDefinedMessageObservation.ClientSelectedElementsWrapper(sepd);
			Assert.AreEqual(sepd,w.ToString());
		}
	}

	[TestFixture]
	public class TraverseDOMTests 
	{
		[Test] public void ForewardTest() 
		{
			string testHtml = @"
<html>
<head><title>test</title></head>
<body>
<div>
<p><i>hallo</i><b>du</b></p>
<table>
<tr><td><h1>Text</h1></td></tr>
<tr><td><h2>smaller<i>then</i></h2></td></tr>
</table>
</div>
<p>
hallo
</p>
<h3>Hey</h3>
</body>
</html>
";
			IHTMLDocument2 htmlDoc = new HTMLDocumentClass();
			htmlDoc.write(new object[] {testHtml});
			htmlDoc.close();
			IHTMLDOMNode bodyNode = (IHTMLDOMNode)htmlDoc.body;
			IHTMLDOMNode tmpNode = bodyNode;
			IHTMLDOMNode retNode = null;
			int steps = 12;
			retNode = UserDefinedMessageObservation.TraverseDOMForeward(ref tmpNode, ref steps, null);
			Assert.AreEqual(0,steps);
			Assert.AreEqual("h2",tmpNode.nodeName.ToLower());
			Assert.AreEqual("h2",retNode.nodeName.ToLower());
			steps = 2;
			tmpNode = bodyNode;
			retNode = UserDefinedMessageObservation.TraverseDOMForeward(ref tmpNode, ref steps, "table");
			Assert.AreEqual("table",tmpNode.nodeName.ToLower());
			Assert.AreEqual(1, steps);
			Assert.AreEqual("h3",retNode.nodeName.ToLower());
			steps = 2;
			tmpNode = bodyNode;
			retNode = UserDefinedMessageObservation.TraverseDOMForeward(ref tmpNode, ref steps, "p");
			Assert.AreEqual("p",tmpNode.nodeName.ToLower());
			Assert.AreEqual(0,steps);
			Assert.AreEqual("p",retNode.nodeName.ToLower());
			steps = 3;
			tmpNode = bodyNode;
			retNode = UserDefinedMessageObservation.TraverseDOMForeward(ref tmpNode, ref steps, "td");
			Assert.AreEqual("td",tmpNode.nodeName.ToLower());
			Assert.AreEqual(1,steps);
			Assert.AreEqual("h3",retNode.nodeName.ToLower());
			steps = 3;
			tmpNode = bodyNode;
			retNode = UserDefinedMessageObservation.TraverseDOMForeward(ref tmpNode, ref steps, "p");
			Assert.AreEqual("p",tmpNode.nodeName.ToLower());
			Assert.AreEqual(1,steps);
			Assert.AreEqual("h3",retNode.nodeName.ToLower());
			steps = 1;
			tmpNode = bodyNode;
			retNode = UserDefinedMessageObservation.TraverseDOMForeward(ref tmpNode, ref steps, "script");
			Assert.IsNull(tmpNode);
			Assert.AreEqual(1,steps);
			Assert.AreEqual("h3",retNode.nodeName.ToLower());
		}
	}

	[TestFixture]
	public class CheckClientSelectedElementsPositionDescriptorTests
	{
		[Test] public void SelfTest1() 
		{
			string userSelectionPath = "0;1;p,1,3,101,2,-1;>1;1;p,2,2,107,3,-1;>";
			string[] parts1 = userSelectionPath.Split(new char[] {'>'});
			Assert.AreEqual(3,parts1.Length);
			Assert.AreEqual("0;1;p,1,3,101,2,-1;",parts1[0]);
			Assert.AreEqual("1;1;p,2,2,107,3,-1;",parts1[1]);
			Assert.AreEqual("",parts1[2]);
		}

		[Test] public void SelfTest2() 
		{
			string userSelectionPath = "a";
			string[] parts1 = userSelectionPath.Split(new char[] {'>'});
			Assert.AreEqual(1,parts1.Length);
		}
		
		
		[Test] public void CorrectTest1() 
		{
			string errorMessage;
			bool result = UserDefinedMessageObservation.CheckClientSelectedElementsPositionDescriptor("0;1;p,1,3,101,2,-1;>1;1;p,2,2,107,3,-1;>", out errorMessage);
			Assert.IsTrue(result);
		}
	} // end: class SetSelectedElementsPositionDescriptorTests
}
