namespace WebseitenMonitor.Tests
{
	using System;
	using NUnit.Framework;
	using WebseitenMonitor.DomainObjects.Behaviour;
	using WebseitenMonitor;
	using System.Collections;
	using System.IO;
	using System.Text;
	using WebseitenMonitor.HtmlUtilities;
	using WebseitenMonitor.UserSelection;
	using WebseitenMonitor.UriUtilities;


	[TestFixture]
	public class HtmlTagScannerTests 
	{	
		[Test] public void Selftest1() 
		{
			Assert.IsTrue(Char.IsWhiteSpace('\n'));
			Assert.IsTrue(Char.IsWhiteSpace('\r'));
			Assert.IsTrue(Char.IsWhiteSpace('\t'));
			Assert.IsTrue(Char.IsWhiteSpace(' '));
			Assert.IsFalse(Char.IsWhiteSpace('\u0080')); // eof-Char used by scanner
		}

		[Test] public void Test1() 
		{
			StringBuilder sb = new StringBuilder(@"<P><B>Thema 5: Webseiten-Monitor</B><BR>Es soll ein Werkzeug  werden.<BR><I>Team:</I> Manuel Wallnfer (0155478)<BR><I>Teamgre</I>: 2
Personen<BR><I>Betreuer</I>: <A href=""http://www.ssw.uni-linz.ac.at/General/Staff/TK"">Thomas Kotzmann</A> </P>");
			HtmlTagScanner scanner = new HtmlTagScanner();
			scanner.Init(sb);
			HtmlTagToken t = scanner.NextTag();
			Assert.AreEqual(HtmlTagToken.openingTag, t.kind);
			Assert.AreEqual("p",t.name);
			Assert.AreEqual(0,t.beginPos);
			Assert.AreEqual(2,t.endPos);
			t = scanner.NextTag();
			Assert.AreEqual(HtmlTagToken.openingTag, t.kind);
			Assert.AreEqual("b",t.name);
			Assert.AreEqual(3,t.beginPos);
			Assert.AreEqual(5,t.endPos);
			t = scanner.NextTag();
			Assert.AreEqual(HtmlTagToken.closingTag, t.kind);
			Assert.AreEqual("b",t.name);
			Assert.AreEqual(32,t.beginPos);
			Assert.AreEqual(35,t.endPos);
			t = scanner.NextTag();
			Assert.AreEqual(HtmlTagToken.openingTag, t.kind);
			Assert.AreEqual("br",t.name);
			Assert.AreEqual(36,t.beginPos);
			Assert.AreEqual(39,t.endPos);
			t = scanner.NextTag();t = scanner.NextTag();t = scanner.NextTag();t = scanner.NextTag();
			t = scanner.NextTag();t = scanner.NextTag();t = scanner.NextTag();t = scanner.NextTag();
			t = scanner.NextTag();t = scanner.NextTag();
			Assert.AreEqual(HtmlTagToken.openingTag,t.kind);
			Assert.AreEqual("a",t.name);
			Assert.AreEqual(166,t.beginPos);
			Assert.AreEqual(222,t.endPos);
			t = scanner.NextTag();
			Assert.AreEqual(HtmlTagToken.closingTag,t.kind);
			Assert.AreEqual("a",t.name);
			Assert.AreEqual(238,t.beginPos);
			Assert.AreEqual(241,t.endPos);
			t = scanner.NextTag();
			Assert.AreEqual(HtmlTagToken.closingTag,t.kind);
			Assert.AreEqual("p",t.name);
			Assert.AreEqual(243,t.beginPos);
			Assert.AreEqual(246,t.endPos);
			t = scanner.NextTag();
			Assert.AreEqual(HtmlTagToken.eof, t.kind);
			t = scanner.NextTag();
			Assert.AreEqual(HtmlTagToken.eof, t.kind);
		}

		[Test] public void Test2() 
		{
			StringBuilder sb = new StringBuilder("--<h1>ajbia<br/>kkjdi</H1>");
			HtmlTagScanner scanner = new HtmlTagScanner();
			scanner.Init(sb);
			HtmlTagToken t = scanner.NextTag();
			Assert.AreEqual(HtmlTagToken.openingTag, t.kind);
			Assert.AreEqual("h1",t.name);
			Assert.AreEqual(2,t.beginPos);
			Assert.AreEqual(5,t.endPos);
			t = scanner.NextTag();
			Assert.AreEqual(HtmlTagToken.bothTag, t.kind);
			Assert.AreEqual("br",t.name);
			Assert.AreEqual(11,t.beginPos);
			Assert.AreEqual(15,t.endPos);
			t = scanner.NextTag();
			Assert.AreEqual(HtmlTagToken.closingTag, t.kind);
			Assert.AreEqual("h1",t.name);
			Assert.AreEqual(21,t.beginPos);
			Assert.AreEqual(25,t.endPos);
		}
	} // end class HtmlScannerTests
	
	
	[TestFixture]
	public class HtmlTagAttributeScannerTests 
	{
		[Test] public void BasicTest1() 
		{
			string s = "<a href=\"http://user:pass@192.168.0.0:23/appdir/test.html\">";
			HtmlTagAttributeScanner scanner = new HtmlTagAttributeScanner();
			scanner.Init(s);
			HtmlTagAttributeToken t = scanner.NextAttribute();
			Assert.AreEqual(HtmlTagAttributeToken.quotedAttribute, t.kind);
			Assert.AreEqual("href", t.name);
			Assert.AreEqual("http://user:pass@192.168.0.0:23/appdir/test.html", t.val);
			Assert.AreEqual(3, t.nameBeginPos);
			Assert.AreEqual(7, t.nameOuterEndPos);
			Assert.AreEqual(9, t.valBeginPos);
			Assert.AreEqual(57, t.valOuterEndPos);
			t = scanner.NextAttribute();
			Assert.AreEqual(HtmlTagAttributeToken.eof, t.kind);
		}

		[Test] public void BasicTest2() 
		{
			string s = "<a href=http://user:pass@192.168.0.0:23/appdir/test.html>";
			HtmlTagAttributeScanner scanner = new HtmlTagAttributeScanner();
			scanner.Init(s);
			HtmlTagAttributeToken t = scanner.NextAttribute();
			Assert.AreEqual(HtmlTagAttributeToken.nonQuotedAttribute, t.kind);
			Assert.AreEqual("href", t.name);
			Assert.AreEqual("http://user:pass@192.168.0.0:23/appdir/test.html", t.val);
			Assert.AreEqual(3, t.nameBeginPos);
			Assert.AreEqual(7, t.nameOuterEndPos);
			Assert.AreEqual(8, t.valBeginPos);
			Assert.AreEqual(56, t.valOuterEndPos);
			t = scanner.NextAttribute();
			Assert.AreEqual(HtmlTagAttributeToken.eof, t.kind);
		}

		[Test] public void BasicTest3()
		{
			string s = "<a href = /other.htm#anker type =	\"text/html\">";
			HtmlTagAttributeScanner scanner = new HtmlTagAttributeScanner();
			scanner.Init(s);
			HtmlTagAttributeToken t = scanner.NextAttribute();
			Assert.AreEqual(HtmlTagAttributeToken.nonQuotedAttribute, t.kind);
			Assert.AreEqual("href", t.name);
			Assert.AreEqual("/other.htm#anker", t.val);
			Assert.AreEqual(3, t.nameBeginPos);
			Assert.AreEqual(7, t.nameOuterEndPos);
			Assert.AreEqual(10, t.valBeginPos);
			Assert.AreEqual(26, t.valOuterEndPos);
			t = scanner.NextAttribute();
			Assert.AreEqual(HtmlTagAttributeToken.quotedAttribute, t.kind);
			Assert.AreEqual("type", t.name);
			Assert.AreEqual("text/html", t.val);
			Assert.AreEqual(27, t.nameBeginPos);
			Assert.AreEqual(31, t.nameOuterEndPos);
			Assert.AreEqual(35, t.valBeginPos);
			Assert.AreEqual(44, t.valOuterEndPos);
			t = scanner.NextAttribute();
			Assert.AreEqual(HtmlTagAttributeToken.eof, t.kind);
		}

		[Test] public void AllowedAttrValueCharTest() 
		{
			string s = "<a href=\"az09-_/\\?#'@%$|;,.:& \t\" >";
			HtmlTagAttributeScanner scanner = new HtmlTagAttributeScanner();
			scanner.Init(s);
			HtmlTagAttributeToken t = scanner.NextAttribute();
			Assert.AreEqual(HtmlTagAttributeToken.quotedAttribute, t.kind);
			Assert.AreEqual("href", t.name);
			Assert.AreEqual("az09-_/\\?#'@%$|;,.:& \t", t.val);
			Assert.AreEqual(3, t.nameBeginPos);
			Assert.AreEqual(7, t.nameOuterEndPos);
			Assert.AreEqual(9, t.valBeginPos);
			Assert.AreEqual(32, t.valOuterEndPos);
			t = scanner.NextAttribute();
			Assert.AreEqual(HtmlTagAttributeToken.eof, t.kind);
		}

		[Test] public void MultiLineTest() 
		{
			string s = @"<a href=""./other.html"" 
     name="""" type=text/comma-separated-values
 >";
			HtmlTagAttributeScanner scanner = new HtmlTagAttributeScanner();
			scanner.Init(s);
			HtmlTagAttributeToken t = scanner.NextAttribute();
			Assert.AreEqual(HtmlTagAttributeToken.quotedAttribute, t.kind);
			Assert.AreEqual("href", t.name);
			Assert.AreEqual("./other.html", t.val);
			Assert.AreEqual(3, t.nameBeginPos);
			Assert.AreEqual(7, t.nameOuterEndPos);
			Assert.AreEqual(9, t.valBeginPos);
			Assert.AreEqual(21, t.valOuterEndPos);
			t = scanner.NextAttribute();
			Assert.AreEqual(HtmlTagAttributeToken.quotedAttribute, t.kind);
			Assert.AreEqual("name", t.name);
			Assert.AreEqual("", t.val);
			Assert.AreEqual(30, t.nameBeginPos);
			Assert.AreEqual(34, t.nameOuterEndPos);
			Assert.AreEqual(36, t.valBeginPos);
			Assert.AreEqual(36, t.valOuterEndPos);
			t = scanner.NextAttribute();
			Assert.AreEqual(HtmlTagAttributeToken.nonQuotedAttribute, t.kind);
			Assert.AreEqual("type", t.name);
			Assert.AreEqual("text/comma-separated-values", t.val);
			Assert.AreEqual(38, t.nameBeginPos);
			Assert.AreEqual(42, t.nameOuterEndPos);
			Assert.AreEqual(43, t.valBeginPos);
			Assert.AreEqual(70, t.valOuterEndPos);
			t = scanner.NextAttribute();
			Assert.AreEqual(HtmlTagAttributeToken.eof, t.kind);
		}

		[Test] public void GoogleBodyTagTest() 
		{
			string s = @"<body bgcolor=#ffffff text=#000000 link=#0000cc vlink=#551a8b alink=#ff0000 onLoad=sf()>";
			HtmlTagAttributeScanner scanner = new HtmlTagAttributeScanner();
			scanner.Init(s);
			HtmlTagAttributeToken t = scanner.NextAttribute();
			Assert.AreEqual(HtmlTagAttributeToken.nonQuotedAttribute, t.kind);
			Assert.AreEqual("bgcolor", t.name);
			Assert.AreEqual("#ffffff", t.val);
			Assert.AreEqual(6, t.nameBeginPos);
			Assert.AreEqual(13, t.nameOuterEndPos);
			Assert.AreEqual(14, t.valBeginPos);
			Assert.AreEqual(21, t.valOuterEndPos);
			t = scanner.NextAttribute();
			Assert.AreEqual(HtmlTagAttributeToken.nonQuotedAttribute, t.kind);
			Assert.AreEqual("text", t.name);
			Assert.AreEqual("#000000", t.val);
			Assert.AreEqual(22, t.nameBeginPos);
			Assert.AreEqual(26, t.nameOuterEndPos);
			Assert.AreEqual(27, t.valBeginPos);
			Assert.AreEqual(34, t.valOuterEndPos);
			t = scanner.NextAttribute();
			Assert.AreEqual(HtmlTagAttributeToken.nonQuotedAttribute, t.kind);
			Assert.AreEqual("link", t.name);
			Assert.AreEqual("#0000cc", t.val);
			Assert.AreEqual(35, t.nameBeginPos);
			Assert.AreEqual(39, t.nameOuterEndPos);
			Assert.AreEqual(40, t.valBeginPos);
			Assert.AreEqual(47, t.valOuterEndPos);
			t = scanner.NextAttribute();
			Assert.AreEqual(HtmlTagAttributeToken.nonQuotedAttribute, t.kind);
			Assert.AreEqual("vlink", t.name);
			Assert.AreEqual("#551a8b", t.val);
			Assert.AreEqual(48, t.nameBeginPos);
			Assert.AreEqual(53, t.nameOuterEndPos);
			Assert.AreEqual(54, t.valBeginPos);
			Assert.AreEqual(61, t.valOuterEndPos);
			t = scanner.NextAttribute();
			Assert.AreEqual(HtmlTagAttributeToken.nonQuotedAttribute, t.kind);
			Assert.AreEqual("alink", t.name);
			Assert.AreEqual("#ff0000", t.val);
			Assert.AreEqual(62, t.nameBeginPos);
			Assert.AreEqual(67, t.nameOuterEndPos);
			Assert.AreEqual(68, t.valBeginPos);
			Assert.AreEqual(75, t.valOuterEndPos);
			t = scanner.NextAttribute();
			Assert.AreEqual(HtmlTagAttributeToken.nonQuotedAttribute, t.kind);
			Assert.AreEqual("onload", t.name);
			Assert.AreEqual("sf()", t.val);
			Assert.AreEqual(76, t.nameBeginPos);
			Assert.AreEqual(82, t.nameOuterEndPos);
			Assert.AreEqual(83, t.valBeginPos);
			Assert.AreEqual(87, t.valOuterEndPos);
			t = scanner.NextAttribute();
			Assert.AreEqual(HtmlTagAttributeToken.eof, t.kind);
		}
	} // end: class AttributeScanner

	[TestFixture]
	public class UserSelectionHtmlPageEnhancerTests 
	{
		[Test] public void SelfTest1() 
		{
			try 
			{
				Uri myUri = new Uri("./myPage.html"); // must be an absolute uri
				Assert.Fail();
			} 
			catch(System.UriFormatException ue) {ue.ToString();}
		}

		
	} // end: class UserSelectionHtmlPageEnhancerTests
	
}