using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using WebseitenMonitor.Ui;
using WebseitenMonitor.Security;
using WebseitenMonitor.DataAccess;
using WebseitenMonitor.DomainObjects.Behaviour;
using log4net;
using System.Text;

namespace WebseitenMonitor.User
{
	/// <summary>
	/// Summary description for CategoryContent.
	/// </summary>
	public class CategoryContent : System.Web.UI.Page
	{
		private static readonly ILog log = LogManager.GetLogger(typeof(CategoryContent));

		protected bool RefreshCategoryNavigation = false;

		// WebControls
		protected Button CreateNewCategoryButton;
		protected Button DeleteCategoryButton;
		protected Button CreateNewObservationButton;
		protected Button CheckCategoryButton;
		protected Button CheckCategoryRecursiveButton;
		protected Button ChangeCategoryNameButton;
		protected DataGrid CategoryObservationsDataGrid;

		private void Page_Load(object sender, System.EventArgs e)
		{
			CategoryNavigationForest myForest = (CategoryNavigationForest)Session["UserForest"];
			// Treat show requests for Nodes in CategoryNavigationForest
			string showIndexString = null;
			showIndexString = (string)this.Request.Params["show"];
			if(showIndexString != null) 
			{
				int showIndex = -1;
				try 
				{
					showIndex = Convert.ToInt32(showIndexString);
				} 
				catch
				{
					Response.Redirect("../User/ErrorPage.aspx", true);
				}
				// Mark selected Category in CategoryNavigation
				myForest.SelectedCategoryId = showIndex;
				this.RefreshCategoryNavigation = true;
				if(showIndex == -1) 
				{
					this.CreateNewObservationButton.Visible = false;
				}
			}
			if(myForest.SelectedCategoryId < 0) 
			{
				this.CreateNewObservationButton.Visible = false;
				this.DeleteCategoryButton.Visible = false;
				this.CheckCategoryButton.Visible = false;
				this.ChangeCategoryNameButton.Visible = false;
			}
			// Check if Refresh of CategoryNavigationView is necessare
			if(Session["NavigationUpdateRequired"] != null && (bool)Session["NavigationUpdateRequired"] == true) 
			{
				this.RefreshCategoryNavigation = true;
				Session["NavigationUpdateRequired"] = false;
			}
			
			this.initializeDataGrid(myForest);
		}

		private void initializeDataGrid(CategoryNavigationForest forest) 
		{
			// initialise DataGrid for Observations in selected Category
			this.CategoryObservationsDataGrid.Visible = true;
			ArrayList myCategoryObservationItems;
			string errorMsg;
			ObservationsMediator.ReadResult res	= ObservationsMediator.ReadCategoryObservationItemListForCategory(forest.SelectedCategoryId,
				out myCategoryObservationItems,
				out errorMsg);
			if(res == ObservationsMediator.ReadResult.Ok) 
			{
				this.CategoryObservationsDataGrid.DataSource = myCategoryObservationItems;
				this.CategoryObservationsDataGrid.DataBind();
				this.CategoryObservationsDataGrid.Visible = (myCategoryObservationItems.Count > 0);
			}
		}

		private void initializeDataGrid(ArrayList myCategoryObservationItems) 
		{
			this.CategoryObservationsDataGrid.DataSource = myCategoryObservationItems;
			this.CategoryObservationsDataGrid.DataBind();
			this.CategoryObservationsDataGrid.Visible = (myCategoryObservationItems.Count > 0);
		}

		protected bool IsRootOrNoneSelected() 
		{
				CategoryNavigationForest myForest = (CategoryNavigationForest)Session["UserForest"];
				return myForest.SelectedCategoryId < 0;
		}
		
		protected string GetObservationKindImageUrl(ObservationKind kind) 
		{
			if(kind == ObservationKind.ChangeObservation) 
				return "../img/ObsKindIconChange.bmp";
			else if(kind == ObservationKind.UserDefinedMessageObservation) 
				return "../img/ObsKindIconMessage.bmp";
			return "";
		}

		protected string GetObservationStateImageUrl(DomainObjects.Behaviour.ChangeStateEnum state) 
		{
			if(state == DomainObjects.Behaviour.ChangeStateEnum.Unchanged) 
				return "../img/ObsStateIconUnchanged.bmp";
			else if(state == DomainObjects.Behaviour.ChangeStateEnum.Changed) 
				return "../img/ObsStateIconChanged.bmp";
			else if(state == DomainObjects.Behaviour.ChangeStateEnum.Error) 
				return "../img/ObsStateIconError.bmp";
			else if(state == DomainObjects.Behaviour.ChangeStateEnum.Init)
				return "../img/ObsStateIconInit.bmp";
			else if(state == DomainObjects.Behaviour.ChangeStateEnum.New)
				return "../img/ObsStateIconNew.bmp";
			return "";
		}

		protected string GetShowInDetailUrl(ObservationKind kind, int Id) 
		{
			if(kind == ObservationKind.ChangeObservation)
				return "./ChangeObservationDetail.aspx?show=" + Id;
			else if(kind == ObservationKind.UserDefinedMessageObservation)
				return "./UserDefinedMessageObservationDetail.aspx?show=" + Id;
			else 
				return "./ErrorPage.aspx";
		}

		protected string GetObservationDate(DateTime date) 
		{
			if(date == Observation.InitDateTime)
				return "-";
			else
				return date.ToShortDateString() + "  " + date.ToShortTimeString();
		}

		protected void DeleteCategoryClicked(object source, EventArgs args) 
		{
			CategoryNavigationForest myForest = (CategoryNavigationForest)Session["UserForest"];
			string errorMsg;
			CategoryNavigationForest.DeleteCategoryResult res = myForest.DeleteCategory(myForest.SelectedCategoryId, out errorMsg);
			this.RefreshCategoryNavigation = true;
			if(myForest.SelectedCategoryId < 0) 
			{
				this.CreateNewObservationButton.Visible = false;
				this.DeleteCategoryButton.Visible = false;
			} 
			else 
			{
				this.initializeDataGrid(myForest);
			}
			this.Response.Redirect("./CategoryContent.aspx?show=" + myForest.SelectedCategoryId);
			log.Debug("SELECTED NODE AFTER DELETE: " + myForest.SelectedCategoryId);

		}

		protected void CreateNewCategoryButtonClicked(object source, EventArgs args) 
		{
			// CreateNewCategory.aspx is responsible for creatin a new Category
			this.Server.Transfer("./CreateNewCategory.aspx");
		}

		protected void ChangeCategoryNameButtonClicked(object source, EventArgs args) 
		{
			this.Server.Transfer("./ChangeCategoryName.aspx");
		}

		protected void CreateNewObservationButtonClicked(object source, EventArgs args) 
		{
			// ObservationCreateWizard.aspx is responsible for creating a new Observation
			// First clear Session-Items responsible for creating new Observation
			if(Session["CreateWizard_KindItem"] != null)
				Session.Remove("CreateWizard_KindItem");
			if(Session["CreateWizard_NameItem"] != null)
				Session.Remove("CreateWizard_NameItem");
			if(Session["CreateWizard_UrlItem"] != null)
				Session.Remove("CreateWizard_UrlItem");
			if(Session["CreateWizard_DepthItem"] != null)
				Session.Remove("CreateWizard_DepthItem");
			if(Session["CreateWizard_MsgObsGuiCreateMode"] != null)
				Session.Remove("CreateWizard_MsgObsGuiCreateMode");
			if(Session["CreateWizard_SearchPatternEnableChooseItem"] != null)
				Session.Remove("CreateWizard_SearchPatternEnableChooseItem");
			if(Session["CreateWizard_SearchPatternItem"] != null)
				Session.Remove("CreateWizard_SearchPatternItem");
			if(Session["CreateWizard_MessageTemplateItem"] != null)
				Session.Remove("CreateWizard_MessageTemplateItem");
			if(Session["CreateWizard_SelectedElementsPositionDescriptorItem"] != null)
				Session.Remove("CreateWizard_SelectedElementsPositionDescriptorItem");
			this.Response.Redirect("./ObservationCreateWizard_General.aspx");
			//this.Server.Transfer("./ObservationCreateWizard_General.aspx");
		}

		protected void CheckCategoryButtonClicked(object sender, EventArgs args) 
		{
			ArrayList myCategoryObservationItems = (ArrayList)this.CategoryObservationsDataGrid.DataSource;
			int newCheckCounterValue = ((int)Session["UserCheckCounter"]) + 1;
			// Update users CheckCounter in database 
			UserDataWrapper udw = new UserDataWrapper(User.Identity.Name);
			string errorMessage;
			UsersMediator.UpdateResult ccUpdRes = UsersMediator.UpdateUsersCheckCounter(
				udw.Userid, 
				newCheckCounterValue, 
				out errorMessage);
			if(ccUpdRes != UsersMediator.UpdateResult.Ok) 
			{
				Session["ErrorMessage"] = "Beim Aktualisieren der Datenbank ist ein Problem aufgetreten.";
				Response.Redirect("./ErrorPage.aspx");
				return;
			}
			// Update Users CheckCounter in Session
			Session["UserCheckCounter"] = newCheckCounterValue;
			bool problemOccured;
			this.checkCategory(
				myCategoryObservationItems, 
				newCheckCounterValue,
				out problemOccured, 
				out errorMessage);

			if(problemOccured) 
			{
				Session["ErrorMessage"] = "W&auml;hrend der &Uuml;berpr&uuml;fung sind folgende Probleme aufgetreten:<br>" + errorMessage;
				Response.Redirect("./ErrorPage.aspx");
				return;
			}
			else 
			{
				// assertion: Check was successfull foreach item
				// refresh view
				this.initializeDataGrid(myCategoryObservationItems);
			}
		} // end: method CheckCategoryBottonClicked

		protected void CheckCategoryRecursiveButtonClicked(object sender, EventArgs args) 
		{
			StringBuilder errorText = new StringBuilder();
			errorText.Append("W&auml;hrend der &Uuml;berpr&uuml;fung sind folgende Probleme aufgetreten:<br>"); 
			bool problemOccured = false;
			bool tmpProblemOccured;
			string errorMessage;
			// Determine selected Category and descendent categories
			CategoryNavigationForest myForest = (CategoryNavigationForest)Session["UserForest"];
			CategoryNavigationForest.CategoryNode myNode = myForest.SelectedCategory;
			ArrayList curChildNodes = null;
			if(myForest.SelectedCategoryId == -1)
				curChildNodes = myForest.RootNodes;
			else if(myNode != null)
				curChildNodes = myNode.ChildNodes;
			else 
			{
				Session["ErrorMessage"] = "Es konnte keine passende Kategorie identifiziert werden!";
				Response.Redirect("./ErrorPage.aspx");
				return;
			}
			int newCheckCounterValue = ((int)Session["UserCheckCounter"]) + 1;
			// Update users CheckCounter in database 
			UserDataWrapper udw = new UserDataWrapper(User.Identity.Name);
			UsersMediator.UpdateResult ccUpdRes = UsersMediator.UpdateUsersCheckCounter(
				udw.Userid, 
				newCheckCounterValue, 
				out errorMessage);
			if(ccUpdRes != UsersMediator.UpdateResult.Ok) 
			{
				Session["ErrorMessage"] = "Beim Aktualisieren der Datenbank ist ein Problem aufgetreten.";
				Response.Redirect("./ErrorPage.aspx");
				return;
			}
			// Update Users CheckCounter in Session
			Session["UserCheckCounter"] = newCheckCounterValue;
			ArrayList nextCurChildNodes = new ArrayList();
			// check observations in selected category if this is not the root
			if(myForest.SelectedCategoryId >= 0) // not invoked at root Category 
			{
				// check Observations directly in selectedCategory
				ArrayList myCategoryObservationItems = (ArrayList)this.CategoryObservationsDataGrid.DataSource;
				this.checkCategory(
					myCategoryObservationItems,
					newCheckCounterValue,
					out tmpProblemOccured,
					out errorMessage);
				if(tmpProblemOccured) 
				{
					problemOccured = true;
					errorText.Append(myNode.CategoryName + ": <br>");
					errorText.Append(errorMessage);
				} 
				else 
				{
					// assertion: Check of Observations in directly selected Category was successfull foreach item
					// refresh view
					this.initializeDataGrid(myCategoryObservationItems);
				}
			}
			// check all observations in descendent categories
			while(curChildNodes != null && curChildNodes.Count > 0) 
			{
				foreach(CategoryNavigationForest.CategoryNode curNode in curChildNodes) 
				{
					this.checkCategory(
						curNode, 
						newCheckCounterValue,
						out tmpProblemOccured, 
						out errorMessage);
					if(tmpProblemOccured) 
					{
						problemOccured = true;
						errorText.Append(curNode.CategoryName + ": <br>");
						errorText.Append(errorMessage);
					}
					nextCurChildNodes.AddRange(curNode.ChildNodes);
				}
				curChildNodes = nextCurChildNodes;
				nextCurChildNodes = new ArrayList();
			}
			if(problemOccured) 
			{
				Session["ErrorMessage"] = errorText.ToString();
				Response.Redirect("./ErrorPage.aspx");
				return;
			}
		} // end: method CheckCategoryRecursiveButtonClicked

		private void checkCategory(
			CategoryNavigationForest.CategoryNode node, 
			int newCheckCounter,
			out bool problemOccured, 
			out string errorMessage) 
		{
			ArrayList myCategoryObservationItems;
			string errorMsg;
			ObservationsMediator.ReadResult res	= ObservationsMediator.ReadCategoryObservationItemListForCategory(
				node.Id,
				out myCategoryObservationItems,
				out errorMsg);
			if(res != ObservationsMediator.ReadResult.Ok) 
			{
				problemOccured = true;
				errorMessage = "Der Inhalt der Kategorie namens \"" + node.CategoryName + "\" konnte nicht ordungsgem&auml;&szlig; aus der Datenbank gelesen werden!";
				return;
			}
			this.checkCategory(
				myCategoryObservationItems,
				newCheckCounter,
				out problemOccured,
				out errorMessage);
		}
		
		// This method checks all Observations in the category specified by its Observations as ArrayList of ObservationItems.
		// Note: The Properties of the ObservationItems get updated according to the check-result.
		private void checkCategory(
			ArrayList categoryObservationItems,
			int newCheckCounter,
			out bool problemOccured,
			out string errorMessage) 
		{
			bool tmpProblemOccured = false;
			StringBuilder errorText = new StringBuilder();
			string errorMsg;
			foreach(ObservationsMediator.CategoryObservationItem item in categoryObservationItems) 
			{
				Observation obs = null;
				// load Observation from database - different kind need different load-method
				if(item.Kind == ObservationKind.ChangeObservation) 
				{
					ChangeObservation chObs;
					ChangeObservationsMediator.ReadResult readRes = ChangeObservationsMediator.LoadChangeObservation(
						item.Id,
						out chObs,
						out errorMsg);
					obs = chObs;
					if(readRes != ChangeObservationsMediator.ReadResult.Ok) 
					{
						tmpProblemOccured = true;
						errorText.Append("Der Beobachtungskanal namens \"" + item.Name + "\" konnte nicht ordnugsgem&auml;&szlig; von der Datenbank gelesen werden!<br>");
						continue;
					}
				}
				else // item.Kind == ObservationKind.UserDefinedMessageObservation 
				{
					UserDefinedMessageObservation udmObs;
					UserDefinedMessageObservationMediator.ReadResult readRes = UserDefinedMessageObservationMediator.LoadUserDefinedMessageObservation(
						item.Id,
						out udmObs, 
						out errorMsg);
					obs = udmObs;
					if(readRes != UserDefinedMessageObservationMediator.ReadResult.Ok) 
					{
						tmpProblemOccured = true;
						errorText.Append("Der Beobachtungskanal namens \"" + item.Name + "\" konnte nicht ordnugsgem&auml;&szlig; von der Datenbank gelesen werden!<br>");
						continue;
					}
				}
				// assertion: obs holds the Observation loaded from the database corresponding to item.id
				CheckResult checkRes = obs.Check(newCheckCounter, out errorMsg);
				if(checkRes != CheckResult.Ok) 
				{
					tmpProblemOccured = true;
					errorText.Append("Der Beobachtungskanal namens \"" + item.Name + "\" konnte nicht ordnugsgem&auml;&szlig; &uuml;berpr&uuml;ft werden!<br>");
					continue;
				}
				// assertion: obs successfully checked
				if(obs is ChangeObservation) 
				{
					ChangeObservationsMediator.UpdateResult updRes = ChangeObservationsMediator.UpdateCheckedChangeObservation(
						(ChangeObservation)obs, 
						out errorMsg);
					if(updRes != ChangeObservationsMediator.UpdateResult.Ok) 
					{
						tmpProblemOccured = true;
						errorText.Append("Der Beobachtungskanl namens \"" + item.Name + "\" konnte nicht ordnungsgem&auml;&szlig; in die Datenbank geschrieben werden!<br>");
						continue;
					}
				} 
				else // obs is UserDefinedMessageObservation 
				{
					UserDefinedMessageObservationMediator.UpdateResult updRes = UserDefinedMessageObservationMediator.UpdateCheckedUserDefinedMessageObservation(
						(UserDefinedMessageObservation)obs, 
						out errorMsg);
					if(updRes == UserDefinedMessageObservationMediator.UpdateResult.Ok) 
					{
						// update item-properties
						item.Date = obs.Date;
						item.ChangeState = obs.ChangeState;
					}
					else
					{
						tmpProblemOccured = true;
						errorText.Append("Der Beobachtungskanl namens \"" + item.Name + "\" konnte nicht ordnungsgem&auml;&szlig; in die Datenbank geschrieben werden!<br>");
						continue;
					}
				}
			} // end: foreach
			if(tmpProblemOccured) 
			{
				problemOccured = true;
				errorMessage = errorText.ToString();
				return;
			}
			problemOccured = false;
			errorMessage = null;
			return;
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Load += new System.EventHandler(this.Page_Load);
		}
		#endregion
	}
}
