using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using WebseitenMonitor.Security;
using WebseitenMonitor.Ui;
using log4net;

namespace WebseitenMonitor.User
{
	/// <summary>
	/// Summary description for CategoryNavigation.
	/// </summary>
	public class CategoryNavigation : System.Web.UI.Page
	{
		private static readonly ILog log = LogManager.GetLogger(typeof(CategoryNavigation));

		protected Label TitleLabel;

		private void Page_Load(object sender, System.EventArgs e)
		{
			TitleLabel.Text = "Kategorien ";
			// Put user code to initialize the page here
			UserDataWrapper curUser = new UserDataWrapper(this.User.Identity.Name);
			// Get CategoryNavigationForest from Session, or load it initially if not present in Session-state
			CategoryNavigationForest myForest = (CategoryNavigationForest)Session["UserForest"];
			if(myForest == null) 
			{
				myForest = new CategoryNavigationForest(curUser.Userid);
				myForest.LoadWholeForest();
				Session["UserForest"] = myForest;
			}
			// Treat open/close requests for Nodes in CategoryNavigationForest
			string openIndexString;
			string closeIndexString;
			if((openIndexString = (string)this.Request.Params["open"]) != null) 
			{
				try 
				{
					myForest.OpenCategory(Convert.ToInt32(openIndexString));
				} 
				catch(Exception ex) 
				{
					log.Error("Couldn't convert querystring open-parameter value " + openIndexString + " to Int32",ex);
				}
			} 
			else if((closeIndexString = (string)this.Request.Params["close"]) != null) 
			{
				try 
				{
					myForest.CloseCategory(Convert.ToInt32(closeIndexString));
				} 
				catch(Exception ex) 
				{
					log.Error("Couldn't convert querystring close-parameter value " + closeIndexString + " to Int32",ex);
				}
			}
		}
		
		// This method is called in the corresponding .aspx File to provide the Page-html-content for the
		// CategoryNavigationForest stored in the Session-state.
		protected void RenderNavigationForest() 
		{
			CategoryNavigationForest myForest = (CategoryNavigationForest)Session["UserForest"];
			if(myForest != null)
				Response.Write(myForest.GenerateHtmlRepresentation(
					"../img/Folder_Selected.bmp",
					"../img/Folder_Unselected.bmp",
					"../img/Plus.bmp","../img/Minus.bmp",
					"../img/Neutral.bmp",
					"./CategoryNavigation.aspx",
					"./CategoryContent.aspx"));
			else
				log.Error("No CategoryNavigationForest to render found in session-state!");
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Load += new System.EventHandler(this.Page_Load);
		}
		#endregion
	}
}
