using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using WebseitenMonitor.DataAccess;
using WebseitenMonitor.DomainObjects.Behaviour;
using WebseitenMonitor.Ui;
using log4net;
using System.Text;
using WebseitenMonitor.Security;

namespace WebseitenMonitor.User
{
	/// <summary>
	/// Summary description for ChangeObservationDetail.
	/// </summary>
	public class ChangeObservationDetail : System.Web.UI.Page
	{
		private static readonly ILog log = LogManager.GetLogger(typeof(ChangeObservationDetail));

		protected Label NameLabel;
		protected Label UrlLabel;
		protected Label DateLabel;
		protected Label ChangeStateLabel;
		protected System.Web.UI.WebControls.Button DeleteButton;
		protected System.Web.UI.WebControls.Button SettingsButton;
		protected System.Web.UI.WebControls.Button CheckButton;
		protected System.Web.UI.WebControls.CheckBox UnchangedFilter;
		protected System.Web.UI.WebControls.CheckBox ChangedFilter;
		protected System.Web.UI.WebControls.CheckBox NewFilter;
		protected System.Web.UI.WebControls.CheckBox InvalidMimeTypeFilter;
		protected System.Web.UI.WebControls.CheckBox UnretrievableFilter;
		protected System.Web.UI.WebControls.CheckBox NotHashableFilter;
		protected Label UnchangedLegendLabel;
		protected Label ChangedLegendLabel;
		protected Label NewLegendLabel;
		protected Label InvalidMimeTypeLegendLabel;
		protected Label UnretrievableLegendLabel;
		protected Label NotHashableLegendLabel;
		protected Label DepthLabel;
		
		private void Page_Load(object sender, System.EventArgs e)
		{
			// Retrieve Observation-id from show-Http-Parameter or viewstate if postback
			int id;
			if(Request.Params["show"] != null) 
			{
				try { id = Convert.ToInt32(Request.Params["show"]); } 
				catch 
				{
					Session["ErrorMessage"] = "Die Seite wurde ung&uuml;ltig parametrisiert.";
					Response.Redirect("./ErrorPage.aspx");
					return;
				}
			}
			else 
			{
				if(Session["ObservedDocumentsTree"] != null)
					id = ((ObservedDocumentsTree)Session["ObservedDocumentsTree"]).Id;
				else 
				{
					Session["ErrorMessage"] = "Es konnte kein Beobachtungskanal identifiziert werden.";
					Response.Redirect("./ErrorPage.aspx");
					return;
				}
			}
			ObservedDocumentsTree obsTree;
			ChangeObservation obs;
			string errorMsg;
			if(Session["ObservedDocumentsTree"] == null || ((ObservedDocumentsTree)Session["ObservedDocumentsTree"]).Id != id) 
			{
				ChangeObservationsMediator.ReadResult res = ChangeObservationsMediator.LoadChangeObservation(id, out obs, out errorMsg);
				if(res == ChangeObservationsMediator.ReadResult.Ok) 
				{
					obsTree = new ObservedDocumentsTree(obs);
					Session["ObservedDocumentsTree"] = obsTree;
					log.Debug("obsTree.Id: " + obsTree.Id + "obsTree.ChangeState: " + obsTree.ChangeState + " " + (int)obsTree.ChangeState);
				} 
				else 
				{
					Session["ErrorMessage"] = "Beim Lesen des Beobachtungskanals aus der Datenbank ist ein Problem aufgetreten.";
					Response.Redirect("./ErrorPage.aspx");
					return;
				}
			} 
			else 
			{
				obsTree = (ObservedDocumentsTree)Session["ObservedDocumentsTree"];
			}
			// assertion: obsTree holds the current ObservedDocumentsTree object of interest
			this.initGeneralPropertiesView(obsTree);
			this.initFilterView();
			// Treat open/close Requests
			if(Request.Params["open"] != null) 
			{
				try 
				{
					int openIndex = Convert.ToInt32(Request.Params["open"]);
					obsTree.OpenDocumentChilds(openIndex);
				} 
				catch 
				{
					Session["ErrorMessage"] = "Die Seite wurde ung&uuml;ltig parametrisiert.";
					Response.Redirect("./ErrorPage.aspx");
					return;
				}
			}
			if(Request.Params["close"] != null) 
			{
				try 
				{
					int closeIndex = Convert.ToInt32(Request.Params["close"]);
					obsTree.CloseDocumentChilds(closeIndex);
				} 
				catch 
				{
					Session["ErrorMessage"] = "Die Seite wurde ung&uuml;ltig parametrisiert.";
					Response.Redirect("./ErrorPage.aspx");
					return;
				}
			}
			if(Request.Params["openrec"] != null) 
			{
				try 
				{
					int openIndex = Convert.ToInt32(Request.Params["openrec"]);
					obsTree.OpenDocumentDescendentsOrSelf(openIndex);
				} 
				catch 
				{
					Session["ErrorMessage"] = "Die Seite wurde ung&uuml;ltig parametrisiert.";
					Response.Redirect("./ErrorPage.aspx");
					return;
				}
			}
			if(Request.Params["closerec"] != null) 
			{
				try 
				{
					int closeIndex = Convert.ToInt32(Request.Params["closerec"]);
					obsTree.CloseDocumentDescendentsOrSelf(closeIndex);
				} 
				catch 
				{
					Session["ErrorMessage"] = "Die Seite wurde ung&uuml;ltig parametrisiert.";
					Response.Redirect("./ErrorPage.aspx");
					return;
				}
			}
			if(!IsPostBack) 
			{
				this.UnchangedFilter.Checked = obsTree.ReadFilterPattern(ObservedDocumentsTree.DescendentOrSelfChangeStateAggregation.Unchanged);
				this.ChangedFilter.Checked = obsTree.ReadFilterPattern(ObservedDocumentsTree.DescendentOrSelfChangeStateAggregation.Changed);
				this.NewFilter.Checked = obsTree.ReadFilterPattern(ObservedDocumentsTree.DescendentOrSelfChangeStateAggregation.New);
				this.InvalidMimeTypeFilter.Checked = obsTree.ReadFilterPattern(ObservedDocumentsTree.DescendentOrSelfChangeStateAggregation.InvalidMimeType);
				this.UnretrievableFilter.Checked = obsTree.ReadFilterPattern(ObservedDocumentsTree.DescendentOrSelfChangeStateAggregation.Unretrievable);
				this.NotHashableFilter.Checked = obsTree.ReadFilterPattern(ObservedDocumentsTree.DescendentOrSelfChangeStateAggregation.NotHashable);
			}
		}

		private void initGeneralPropertiesView(ObservedDocumentsTree obs) 
		{
			this.NameLabel.Text = obs.Name;
			this.UrlLabel.Text = obs.Url.ToString();
			if(obs.ChangeState == ChangeStateEnum.Init) 
			{
				this.DateLabel.Text = "&Uuml;ber diesen Kanal wurde noch keine Beobachtung angesto&szlig;en.";
				this.ChangeStateLabel.Text = "Initialisiert";
			} 
			else 
			{
				this.DateLabel.Text = obs.Date.ToShortDateString() + "  " + obs.Date.ToShortTimeString();
				if(obs.ChangeState == ChangeStateEnum.Changed)
					this.ChangeStateLabel.Text = "Ver&auml;ndert";
				else if(obs.ChangeState == ChangeStateEnum.Unchanged) 
					this.ChangeStateLabel.Text = "Unver&auml;ndert";
				else if(obs.ChangeState == ChangeStateEnum.Error)
					this.ChangeStateLabel.Text = "&Uuml;berpr&uuml;fungsfehler aufgetreten";
				else if(obs.ChangeState == ChangeStateEnum.New)
					this.ChangeStateLabel.Text = "Neu";
				else
					this.ChangeStateLabel.Text = "-";
			}
			this.DepthLabel.Text = obs.Depth.ToString();
		}

		private void initFilterView() 
		{
			this.UnchangedLegendLabel.Style.Add("COLOR", ObservedDocumentsTree.unchangedColor);
			this.UnchangedLegendLabel.Style.Add("BACKGROUND-COLOR", ObservedDocumentsTree.unchangedBackColor);
			this.ChangedLegendLabel.Style.Add("COLOR", ObservedDocumentsTree.changedColor);
			this.ChangedLegendLabel.Style.Add("BACKGROUND-COLOR", ObservedDocumentsTree.changedBackColor);
			this.NewLegendLabel.Style.Add("COLOR", ObservedDocumentsTree.newColor);
			this.NewLegendLabel.Style.Add("BACKGROUND-COLOR", ObservedDocumentsTree.newBackColor);
			this.InvalidMimeTypeLegendLabel.Style.Add("COLOR", ObservedDocumentsTree.invalidMimeTypeColor);
			this.InvalidMimeTypeLegendLabel.Style.Add("BACKGROUND-COLOR", ObservedDocumentsTree.invalidMimeTypeBackColor);
			this.UnretrievableLegendLabel.Style.Add("COLOR", ObservedDocumentsTree.unretrievableColor);
			this.UnretrievableLegendLabel.Style.Add("BACKGROUND-COLOR", ObservedDocumentsTree.unretrievableBackColor);
			this.NotHashableLegendLabel.Style.Add("COLOR", ObservedDocumentsTree.notHashableColor);
			this.NotHashableLegendLabel.Style.Add("BACKGROUND-COLOR", ObservedDocumentsTree.notHashableBackColor);
		}
		
		protected void FilterChanged(object sender, EventArgs args) 
		{
			ObservedDocumentsTree.DescendentOrSelfChangeStateAggregation filter = (ObservedDocumentsTree.DescendentOrSelfChangeStateAggregation)0;
			if(this.UnchangedFilter.Checked)
				filter = filter | ObservedDocumentsTree.DescendentOrSelfChangeStateAggregation.Unchanged;
			if(this.ChangedFilter.Checked)
				filter = filter | ObservedDocumentsTree.DescendentOrSelfChangeStateAggregation.Changed;
			if(this.NewFilter.Checked)
				filter = filter | ObservedDocumentsTree.DescendentOrSelfChangeStateAggregation.New;
			if(this.InvalidMimeTypeFilter.Checked)
				filter = filter | ObservedDocumentsTree.DescendentOrSelfChangeStateAggregation.InvalidMimeType;
			if(this.UnretrievableFilter.Checked)
				filter = filter | ObservedDocumentsTree.DescendentOrSelfChangeStateAggregation.Unretrievable;
			if(this.NotHashableFilter.Checked)
				filter = filter | ObservedDocumentsTree.DescendentOrSelfChangeStateAggregation.NotHashable;
			ObservedDocumentsTree obsTree = (ObservedDocumentsTree)Session["ObservedDocumentsTree"];
			obsTree.SetFilterPattern(filter);
		}

		protected void DeleteButtonClicked(object sender, EventArgs args) 
		{
			ObservedDocumentsTree obsTree = (ObservedDocumentsTree)Session["ObservedDocumentsTree"];
			string errorMsg;
			ObservationsMediator.DeleteResult res = ObservationsMediator.DeleteObservation(obsTree.Id, out errorMsg);
			if(res == ObservationsMediator.DeleteResult.Ok) 
			{
				Session.Remove("ObservedDocumentsTree");
				Response.Redirect("./CategoryContent.aspx");
	
			} 
			else if(res == ObservationsMediator.DeleteResult.InvalidParams) 
			{
				Session["ErrorMessage"] = "Es konnte kein passender zu l&ouml;schender Beobachtungskanal gefunden werden.";
				Response.Redirect("./ErrorPage.aspx");
			} 
			else if(res == ObservationsMediator.DeleteResult.Error)  
			{
				Session["ErrorMessage"] = "Beim L&ouml;schen aus der Datenbank ist ein Problem aufgetreten.";
				Response.Redirect("./ErrorPage.aspx");
			}
		}

		protected void SettingsButtonClicked(object sender, EventArgs args) 
		{
			Response.Redirect("./ChangeObservationEditProperties.aspx");
		}

		protected void CheckButtonClicked(object sender, EventArgs args) 
		{
			ObservedDocumentsTree obsTree = (ObservedDocumentsTree)Session["ObservedDocumentsTree"];
			ChangeObservation obs;
			string errorMsg;
			ChangeObservationsMediator.ReadResult res = ChangeObservationsMediator.LoadChangeObservation(obsTree.Id, out obs, out errorMsg);
			if(res == ChangeObservationsMediator.ReadResult.Ok) 
			{
				int newCheckCounterValue = ((int)Session["UserCheckCounter"]) + 1;
				CheckResult checkRes = obs.Check(newCheckCounterValue, out errorMsg);
				if(checkRes == CheckResult.Ok) 
				{
					// Update users CheckCounter in database 
					UserDataWrapper udw = new UserDataWrapper(User.Identity.Name);
					UsersMediator.UpdateResult ccUpdRes = UsersMediator.UpdateUsersCheckCounter(
						udw.Userid, 
						newCheckCounterValue, 
						out errorMsg);
					if(ccUpdRes != UsersMediator.UpdateResult.Ok) 
					{
						Session["ErrorMessage"] = "Beim Aktualisieren der Datenbank ist ein Problem aufgetreten.";
						Response.Redirect("./ErrorPage.aspx");
						return;
					}
					// Update Users CheckCounter in Session
					Session["UserCheckCounter"] = newCheckCounterValue;
					// Update checked ChangeObservation in database
					ChangeObservationsMediator.UpdateResult updRes = ChangeObservationsMediator.UpdateCheckedChangeObservation(obs,out errorMsg);
					if(updRes != ChangeObservationsMediator.UpdateResult.Ok) 
					{
						Session["ErrorMessage"] = "Beim Aktualisieren der Datenbank ist ein Problem aufgetreten.";
						Response.Redirect("./ErrorPage.aspx");
						return;
					} 
					else 
					{
						// Update ChangeObservation view (ObservedDocumentsTree)
						ObservedDocumentsTree newObsTree = new ObservedDocumentsTree(obs);
						Session["ObservedDocumentsTree"] = newObsTree;
						this.initGeneralPropertiesView(newObsTree);
					}
				} 
				else 
				{
					Session["ErrorMessage"] = "Beim Beobachten ist ein Problem aufgetreten.";
					Response.Redirect("./ErrorPage.aspx");
					return;
				}
			} 
			else 
			{
				Session["ErrorMessage"] = "Beim Lesen des Beobachtungskanals aus der Datenbank ist ein Problem aufgetreten.";
				Response.Redirect("./ErrorPage.aspx");
				return;
			}
		}

		protected void RenderObservedDocumentsTree() 
		{
			ObservedDocumentsTree obsTree = (ObservedDocumentsTree)Session["ObservedDocumentsTree"];
			if(obsTree.ChangeState == ChangeStateEnum.Init) 
			{
				Response.Write("Dieser Beobachtungskanal ist neu initialisiert, es wurde noch keine Beobachtung angesto&szlig;en.<br>" +
					"Der Beobachtungsbaum ist also leer.");
			}
			Response.Write(obsTree.GenerateHtmlRepresentation());
		}
		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion
	}
}
