using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using WebseitenMonitor.Ui;
using log4net;

namespace WebseitenMonitor.User
{
	/// <summary>
	/// Summary description for CreateNewCategoy.
	/// </summary>
	public class CreateNewCategory : System.Web.UI.Page
	{
		private static readonly ILog log = LogManager.GetLogger(typeof(CreateNewCategory));

		// WebControls
		protected TextBox NewCategoryNameTextBox;
		protected Button NewCategoryOkButton;
		protected Button NewCategoryCancelButton;
		protected Label NewCategoryInfoTitleLabel;
		protected Label NewCategoryInfoBodyLabel;
		protected System.Web.UI.WebControls.RequiredFieldValidator CategoryNameRequiredFieldValidator;
		protected CustomValidator CategoryNameCustomValidator;

		private void Page_Load(object sender, System.EventArgs e)
		{
			// Put user code to initialize the page here
		}
		
		// Validates the CategoryName user input.
		protected void ValidateCategoryName(object source, ServerValidateEventArgs args) 
		{
			// Categoryname must at least be one sign long
			if(args.Value == null || args.Value.Length <= 0) 
			{
				((CustomValidator)source).ErrorMessage = "Es muss ein Kategoriename eingegeben werden!";
				args.IsValid = false;
			}
			// Categoryname must not have more than 30 sign's - DB constraint
			if(args.Value.Length > 30) 
			{
				((CustomValidator)source).ErrorMessage = "Der Kategoriename darf maximal 30 Zeichen lang sein!";
				args.IsValid = false;
			}
			// only Letters, Digits, - and _ are allowed
			foreach(char tmp in this.NewCategoryNameTextBox.Text) 
			{
				if(!Char.IsLetterOrDigit(tmp) && !(tmp == '-') && !(tmp == '_')) 
				{
					((CustomValidator)source).ErrorMessage = "Der Kategoriename darf nur aus Buchstaben, Ziffern, &acute;-&acute; und &acute;_&acute; bestehen!";
					args.IsValid = false;
					return;
				}
			}
			// everything ok
			args.IsValid = true;
		}

		protected void NewCategoryOkButtonClicked(object sender, EventArgs args) 
		{
			if(!Page.IsValid)
				return;
			CategoryNavigationForest myForest = (CategoryNavigationForest)Session["UserForest"];
			string errorMsg;
			CategoryNavigationForest.InsertCategoryResult res = myForest.InsertCategory(myForest.SelectedCategoryId, this.NewCategoryNameTextBox.Text, out errorMsg);
			if(res != CategoryNavigationForest.InsertCategoryResult.Ok) 
			{
				this.NewCategoryInfoTitleLabel.Text = "Es ist ein Fehler aufgetreten!";
				this.NewCategoryInfoTitleLabel.Style.Add("COLOR","#ff0000");
				this.NewCategoryInfoBodyLabel.Text = "Die neue Kategorie konnte nicht angelegt werden.";
				this.NewCategoryInfoBodyLabel.Style.Add("COLOR","#ff0000");
			} 
			else 
			{
				myForest.OpenCategory(myForest.SelectedCategoryId);
				Session["NavigationUpdateRequired"] = true;
				this.Response.Redirect("./CategoryContent.aspx");
			}

		}

		protected void NewCategoryCancelButtonClicked(object sender, EventArgs args) 
		{
			this.Response.Redirect("./CategoryContent.aspx");
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion
	}
}
