using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using WebseitenMonitor.Ui.CustomControls;
using WebseitenMonitor.User.Help;
using WebseitenMonitor.Ui;
using WebseitenMonitor.DataAccess;

namespace WebseitenMonitor.User
{
	/// <summary>
	/// Summary description for ObservationCreateWizard_ChangeObsGeneral.
	/// </summary>
	public class ObservationCreateWizard_ChangeObsGeneral : System.Web.UI.Page
	{
		protected System.Web.UI.WebControls.TextBox DepthTextBox;
		protected System.Web.UI.WebControls.RequiredFieldValidator DepthRequiredFieldValidator;
		protected System.Web.UI.WebControls.CustomValidator DepthCustomValidator;
		protected Fold DepthHelpFold;
		protected Button CancelButton;
		protected Button BackButton;
		protected Button CreateButton;
	
		private void Page_Load(object sender, System.EventArgs e)
		{
			if(!this.IsPostBack) 
			{
				// initialize Controlls
				this.DepthTextBox.Text = Session["CreateWizard_DepthItem"]==null?"0":((int)Session["CreateWizard_DepthItem"]).ToString();
				this.DepthHelpFold.AlternateText = HelpTextConstants.DepthHelpHtmlText;
			}
		}

		protected void ValidateDepth(object source, ServerValidateEventArgs args) 
		{
			// Depth of ChangeObservation must at least be one sign long
			if(args.Value == null || args.Value.Length <= 0) 
			{
				((CustomValidator)source).ErrorMessage = "Es muss eine &Uuml;berpr&uuml;fungstiefe eingegeben werden!";
				args.IsValid = false;
			} 
			else
			{
				try 
				{
					int tmpInt = Convert.ToInt32(args.Value);
					if(0 > tmpInt || 3 < tmpInt) 
					{
						((CustomValidator)source).ErrorMessage = "Die &Uuml;berpr&uuml;fungstiefe muss im Bereich 0 bis 3 liegen!";
						args.IsValid = false;
					}
					
				} 
				catch 
				{
					((CustomValidator)source).ErrorMessage = "Die &Uuml;berpr&uuml;fungstiefe muss eine Zahl sein!";
					args.IsValid = false;
				}
			}
		} // end: method ValidateDepth

		protected void CancelButtonClicked(object sender, EventArgs args) 
		{
			if(Session["CreateWizard_KindItem"] != null)
				Session.Remove("CreateWizard_KindItem");
			if(Session["CreateWizard_NameItem"] != null)
				Session.Remove("CreateWizard_NameItem");
			if(Session["CreateWizard_UrlItem"] != null)
				Session.Remove("CreateWizard_UrlItem");
			if(Session["CreateWizard_DepthItem"] != null)
				Session.Remove("CreateWizard_DepthItem");
			if(Session["CreateWizard_MsgObsGuiCreateMode"] != null)
				Session.Remove("CreateWizard_MsgObsGuiCreateMode");
			if(Session["CreateWizard_SearchPatternEnableChooseItem"] != null)
				Session.Remove("CreateWizard_SearchPatternEnableChooseItem");
			if(Session["CreateWizard_SearchPatternItem"] != null)
				Session.Remove("CreateWizard_SearchPatternItem");
			if(Session["CreateWizard_MessageTemplateItem"] != null)
				Session.Remove("CreateWizard_MessageTemplateItem");
			if(Session["CreateWizard_SelectedElementsPositionDescriptorItem"] != null)
				Session.Remove("CreateWizard_SelectedElementsPositionDescriptorItem");
			this.Response.Redirect("./CategoryContent.aspx");

		}

		protected void BackButtonClicked(object sender, EventArgs args) 
		{
			int depth = 0;
			try 
			{
				depth = Int32.Parse(this.DepthTextBox.Text);
			} 
			catch { depth = 0;}
			Session["CreateWizard_DepthItem"] = depth;
			this.Response.Redirect("./ObservationCreateWizard_General.aspx");
		}

		protected void CreateButtonClicked(object sender, EventArgs args) 
		{
			if(this.Page.IsValid) 
			{
				CategoryNavigationForest myForest = (CategoryNavigationForest)Session["UserForest"];
				// Write new (initial) ChangeObservation to database
				string errorMsg;
				int insertedObservationId;
				ChangeObservationsMediator.WriteResult res = ChangeObservationsMediator.WriteNewChangeObservation(
					(string)Session["CreateWizard_NameItem"],
					(Uri)Session["CreateWizard_UrlItem"],
					myForest.SelectedCategoryId,
					Convert.ToInt32(DepthTextBox.Text),
					out insertedObservationId,
					out errorMsg);
				if(res == ChangeObservationsMediator.WriteResult.Ok) 
					this.Response.Redirect("./ChangeObservationDetail.aspx?show=" + insertedObservationId);
				else 
				{
					Session["ErrorMessage"] = "W&auml;hrend dem Anlegen des neuen Beobachtungskanals ist ein Problem beim Datenbankzugriff aufgetreten!<br>" +
						"Es wurde kein neuer Beobachtungskanal angelegt.";
					this.Response.Redirect("./ErrorPage.aspx");
				}
				if(Session["CreateWizard_KindItem"] != null)
					Session.Remove("CreateWizard_KindItem");
				if(Session["CreateWizard_NameItem"] != null)
					Session.Remove("CreateWizard_NameItem");
				if(Session["CreateWizard_UrlItem"] != null)
					Session.Remove("CreateWizard_UrlItem");
				if(Session["CreateWizard_DepthItem"] != null)
					Session.Remove("CreateWizard_DepthItem");
				if(Session["CreateWizard_MsgObsGuiCreateMode"] != null)
					Session.Remove("CreateWizard_MsgObsGuiCreateMode");
				if(Session["CreateWizard_SearchPatternEnableChooseItem"] != null)
					Session.Remove("CreateWizard_SearchPatternEnableChooseItem");
				if(Session["CreateWizard_SearchPatternItem"] != null)
					Session.Remove("CreateWizard_SearchPatternItem");
				if(Session["CreateWizard_MessageTemplateItem"] != null)
					Session.Remove("CreateWizard_MessageTemplateItem");
				if(Session["CreateWizard_SelectedElementsPositionDescriptorItem"] != null)
					Session.Remove("CreateWizard_SelectedElementsPositionDescriptorItem");
			}
		} // end: method CreateButtonClicked

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion
	}
}
