using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using WebseitenMonitor.Ui.CustomControls;
using WebseitenMonitor.User.Help;
using WebseitenMonitor.DomainObjects.Behaviour;

namespace WebseitenMonitor.User
{
	/// <summary>
	/// Summary description for ObservationCreateWizard_General.
	/// </summary>
	public class ObservationCreateWizard_General : System.Web.UI.Page
	{
		protected System.Web.UI.WebControls.TextBox NameTextBox;
		protected System.Web.UI.WebControls.TextBox UrlTextBox;
		protected Fold NameHelpFold;
		protected Fold UrlHelpFold;
		protected Button CancelButton;
		protected System.Web.UI.WebControls.RequiredFieldValidator NameRequiredFieldValidator;
		protected System.Web.UI.WebControls.CustomValidator NameCustomValidator;
		protected System.Web.UI.WebControls.RequiredFieldValidator UrlRequiredFieldValidator;
		protected System.Web.UI.WebControls.CustomValidator UrlCustomValidator;
		protected System.Web.UI.WebControls.RadioButton MsgObsChoosenRadioButton;
		protected System.Web.UI.WebControls.RadioButton ChangeObsChoosenRadioButton;
		protected WebseitenMonitor.Ui.CustomControls.Fold ChooseKindHelpFold;
		protected Button ForwardButton;
	
		private void Page_Load(object sender, System.EventArgs e)
		{
			if(!this.IsPostBack) 
			{
				// initialize Controlls
				if(Session["CreateWizard_KindItem"] == null) 
				{
					this.MsgObsChoosenRadioButton.Checked = true;
					this.ChangeObsChoosenRadioButton.Checked = false;
				} 
				else 
				{
					this.MsgObsChoosenRadioButton.Checked = ((ObservationKind)Session["CreateWizard_KindItem"])==ObservationKind.UserDefinedMessageObservation?true:false;
					this.ChangeObsChoosenRadioButton.Checked = !this.MsgObsChoosenRadioButton.Checked;
				}
				this.NameTextBox.Text = Session["CreateWizard_NameItem"]==null?"":(string)Session["CreateWizard_NameItem"];
				this.UrlTextBox.Text = Session["CreateWizard_UrlItem"]==null?"":((Uri)Session["CreateWizard_UrlItem"]).ToString();
				this.ChooseKindHelpFold.AlternateText = HelpTextConstants.ChooseKindHelpHtmlText;
				this.NameHelpFold.AlternateText = HelpTextConstants.NameHelpHtmlText;
				this.UrlHelpFold.AlternateText = HelpTextConstants.UrlHelpHtmlText;
			}
		} // end: method Page_Load

		protected void ValidateName(object source, ServerValidateEventArgs args) 
		{
			// Name of ChangeObservation must at least be one sign long
			if(args.Value == null || args.Value.Length <= 0) 
			{
				((CustomValidator)source).ErrorMessage = "Es muss ein Name f&uuml;r den Beobachtungskanal eingegeben werden!";
				args.IsValid = false;
			}
				// Name must not have more than 50 sign's - DB constraint
			else if(args.Value.Length > 50) 
			{
				((CustomValidator)source).ErrorMessage = "Der Name des Beobachtungskanals darf maximal 50 Zeichen lang sein!";
				args.IsValid = false;
			}
			else 
			{
				// only Letters, Digits, - and _ are allowed
				foreach(char tmpChar in args.Value) 
				{
					if(!Char.IsLetterOrDigit(tmpChar) && !(tmpChar == '-') && !(tmpChar == '_')) 
					{
						((CustomValidator)source).ErrorMessage = "Der Name des Beobachtungskanals darf nur aus Buchstaben, Ziffern, &acute;-&acute; und &acute;_&acute; bestehen!";
						args.IsValid = false;
						break;
					}
				}
			}
		} // end: method ValidateName

		protected void ValidateUrl(object source, ServerValidateEventArgs args) 
		{
			// Url of ChangeObservation must at least be one sign long
			if(args.Value == null || args.Value.Length <= 0) 
			{
				((CustomValidator)source).ErrorMessage = "Es muss eine der Beobachtung zugrunde liegende Url angegeben werden!";
				args.IsValid = false;
			} 
			else
			{
				try 
				{
					Uri tmpUri = new Uri(args.Value);
				} 
				catch 
				{
					((CustomValidator)source).ErrorMessage = "Die eingegebene Url ist syntaktisch ung&uuml;ltig!";
					args.IsValid = false;
				}
			}
		} // end: method ValidateUrl

		protected void CancelButtonClicked(object sender, EventArgs args) 
		{
			if(Session["CreateWizard_KindItem"] != null)
				Session.Remove("CreateWizard_KindItem");
			if(Session["CreateWizard_NameItem"] != null)
				Session.Remove("CreateWizard_NameItem");
			if(Session["CreateWizard_UrlItem"] != null)
				Session.Remove("CreateWizard_UrlItem");
			if(Session["CreateWizard_DepthItem"] != null)
				Session.Remove("CreateWizard_DepthItem");
			if(Session["CreateWizard_MsgObsGuiCreateMode"] != null)
				Session.Remove("CreateWizard_MsgObsGuiCreateMode");
			if(Session["CreateWizard_SearchPatternEnableChooseItem"] != null)
				Session.Remove("CreateWizard_SearchPatternEnableChooseItem");
			if(Session["CreateWizard_SearchPatternItem"] != null)
				Session.Remove("CreateWizard_SearchPatternItem");
			if(Session["CreateWizard_MessageTemplateItem"] != null)
				Session.Remove("CreateWizard_MessageTemplateItem");
			if(Session["CreateWizard_SelectedElementsPositionDescriptorItem"] != null)
				Session.Remove("CreateWizard_SelectedElementsPositionDescriptorItem");
			this.Response.Redirect("./CategoryContent.aspx");
		} // end: method CancelButtonClicked

		protected void ForwardButtonClicked(object sender, EventArgs args) 
		{
			if(this.Page.IsValid) 
			{
				Session["CreateWizard_KindItem"] = this.MsgObsChoosenRadioButton.Checked==true?ObservationKind.UserDefinedMessageObservation:ObservationKind.ChangeObservation;
				Session["CreateWizard_NameItem"] = this.NameTextBox.Text;
				Session["CreateWizard_UrlItem"] = new Uri(this.UrlTextBox.Text);
				if(this.MsgObsChoosenRadioButton.Checked)
					this.Response.Redirect("./ObservationCreateWizard_MsgObsGeneral.aspx");
				else
					this.Response.Redirect("./ObservationCreateWizard_ChangeObsGeneral.aspx");
			}
		} // end: method ForwardButtonClicked

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion
	}
}
