using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.IO;
using System.Text;
using WebseitenMonitor.DomainObjects.DataSource;
using WebseitenMonitor.UserSelection;
using System.Configuration;
using log4net;

namespace WebseitenMonitor.User.Select
{
	/// <summary>
	/// Summary description for EnhanceSelectedPage.
	/// </summary>
	public class EnhanceSelectedPage : System.Web.UI.Page
	{
		private static readonly ILog log = LogManager.GetLogger(typeof(EnhanceSelectedPage));

		private void Page_Load(object sender, System.EventArgs e)
		{
			// Put user code to initialize the page here
			DataSource ds = new HttpDataSource();
			string url = (string)Session["SelectedPageURL"];
			if(url == null || url.Equals("")) 
			{
				Response.Output.Write("<html><head></head><body><p>Es wurde keine Seite ausgewhlt.</p></body></html>");
				return;
			}
			string errorMessage = "";
			if(!ds.SetSourceURL(url,out errorMessage)) 
			{
				Response.Output.Write("<html><head></head><body><p>Es ist ein Fehler aufgetreten: " + errorMessage + "</p></body></html>");
				return;
			}
			
			TextReader sourceReader;
			DataSource.GetSourceReaderResult getSourceResult = ds.GetSourceReader(out sourceReader, out errorMessage);
			if(getSourceResult != DataSource.GetSourceReaderResult.Ok) 
			{
				Response.Output.Write("<html><head></head><body><p>Es ist ein Fehler aufgetreten: " + errorMessage + "</p></body></html>");
				return;
			}
			// Determine Parameter for page-enhancement
			Uri sourceResponseUri = ds.RetrievedSourceURI;
			if(sourceResponseUri == null) 
			{
				try 
				{
					sourceResponseUri = new Uri(url);
				} 
				catch 
				{
					Response.Output.Write("<html><head></head><body><p>Es ist ein Fehler aufgetreten: Keine absolute Quell URI oder -URL bestimmbar.</p></body></html>");
					return;
				}
			}
			StringBuilder enhancedPage = null;
			string myAbsoluteScriptUriString = null;
			string myAbsoluteScriptFormTargetUriString = null;
			try 
			{
				Uri myAbsoluteScriptUri = new Uri(
					new Uri(ConfigurationSettings.AppSettings["myAppUriAbsolutePathPrefix"] + Request.ApplicationPath + "/"),
					"./User/Select/Script/UserSelectScript.js"
					);
				myAbsoluteScriptUriString = myAbsoluteScriptUri.AbsoluteUri;
				Uri myAbsoluteScriptFormTargetUri = new Uri(
					new Uri(ConfigurationSettings.AppSettings["myAppUriAbsolutePathPrefix"] + Request.ApplicationPath + "/"),
					"./User/UserDefinedObservationSelectTarget.aspx"
					);
				myAbsoluteScriptFormTargetUriString = myAbsoluteScriptFormTargetUri.AbsoluteUri;
				
			} 
			catch 
			{
				Response.Output.Write("<html><head></head><body><p>Es ist ein Fehler aufgetreten: Keine absoluten Script-bezogenen URI's bestimmbar.</p></body></html>");
				return;
			}
			string htmlSource = sourceReader.ReadToEnd();
			UserSelectionHtmlPageEnhancer enhancer = new UserSelectionHtmlPageEnhancer();
			UserSelectionHtmlPageEnhancer.EnhanceHtmlPageResult enhanceResult = enhancer.EnhanceHtmlPage(
				new StringReader(htmlSource),
				sourceResponseUri,
				myAbsoluteScriptUriString,
				myAbsoluteScriptFormTargetUriString,
				out enhancedPage,
				out errorMessage);
			if(enhanceResult != UserSelectionHtmlPageEnhancer.EnhanceHtmlPageResult.Ok) 
			{
				Response.Output.Write("<html><head></head><body><p>Es ist ein Fehler aufgetreten: " + errorMessage + "</p></body></html>");
				return;
			}
			Response.Output.Write(enhancedPage.ToString());
			Session["NewObservationHtmlText"] = htmlSource;
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Load += new System.EventHandler(this.Page_Load);
		}
		#endregion
	}
}
