using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using WebseitenMonitor.DataAccess;
using WebseitenMonitor.DomainObjects.Behaviour;
using WebseitenMonitor.Security;

namespace WebseitenMonitor.User
{
	/// <summary>
	/// Summary description for UserDefinedMessageObservationDetail.
	/// </summary>
	public class UserDefinedMessageObservationDetail : System.Web.UI.Page
	{
		protected Label NameLabel;
		protected Label UrlLabel;
		protected Label DateLabel;
		protected Label ChangeStateLabel;
		protected Button CheckButton;
		protected Button SettingsButton;
		protected Button DeleteButton;
		protected Label ResultStateLabel;
		protected Label ResultTextLabel;
		protected Panel ResultTextPanel;

		private void Page_Load(object sender, System.EventArgs e)
		{
			// Retrieve Observation-id from show-Http-Parameter or session
			int id;
			if(Request.Params["show"] != null) 
			{
				try 
				{ 
					id = Convert.ToInt32(Request.Params["show"]);
					Session["CurrentUserDefinedMessageObservationDetailId"] = id;
				}
				catch 
				{
					Session["ErrorMessage"] = "Die Seite wurde ung&uuml;ltig parametrisiert.";
					Response.Redirect("./ErrorPage.aspx");
					return;
				}
		} 
			else 
			{
				if(Session["CurrentUserDefinedMessageObservationDetailId"] != null)
					id = (int)Session["CurrentUserDefinedMessageObservationDetailId"];
				else 
				{
					Session["ErrorMessage"] = "Es konnte kein Beobachtungskanal identifiziert werden.";
					Response.Redirect("./ErrorPage.aspx");
					return;
				}
			}
			// assertion: id holds the id of the UserDefinedMessageObservation to show in detail
			// load corresponding UserDefinedMessageObservation object from Database
			UserDefinedMessageObservation udmObs;
			string errorMsg;
			UserDefinedMessageObservationMediator.ReadResult res = UserDefinedMessageObservationMediator.LoadUserDefinedMessageObservation(
				id,
				out udmObs,
				out errorMsg);
			if(res != UserDefinedMessageObservationMediator.ReadResult.Ok) 
			{
				Session["ErrorMessage"] = "Beim Laden aus der Datenbank ist ein Problem aufgetreten!";
				Response.Redirect("./ErrorPage.aspx");
			}
			// show general properties of UserDefinedMessageObservation
			this.initGeneralPropertiesView(udmObs);
			this.initDetailView(udmObs);
		}

		private void initDetailView(UserDefinedMessageObservation udmObs) 
		{
			switch(udmObs.ResultState) 
			{
				case UserDefinedMessageObservation.ResultStateEnum.Undefined:
					this.ResultStateLabel.Text = "&Uuml;ber diesen Kanal wurde noch keine Beobachtung angesto&szlig;en!";
					this.ResultStateLabel.Style["COLOR"] = "#ff0000";
					this.ResultTextLabel.Text = "";
					break;
				case UserDefinedMessageObservation.ResultStateEnum.Ok:
					this.ResultStateLabel.Text = "Ergebnis wurde erfolgreich bestimmt!";
					this.ResultStateLabel.Style["COLOR"] = "#00aa00";
					this.ResultTextLabel.Text = udmObs.ResultText;
					break;
				case UserDefinedMessageObservation.ResultStateEnum.NoSelectionMatch:
					this.ResultStateLabel.Text = "In der beobachteten Seite konnte keine &Uuml;bereinstimmung mit der definierten Selektion dieses Beobachtungskanals gefunden werden!";
					this.ResultStateLabel.Style["COLOR"] = "#ff0000";
					this.ResultTextLabel.Text = "";
					break;
				case UserDefinedMessageObservation.ResultStateEnum.NoSearchPatternMatch:
					this.ResultStateLabel.Text = "In der beobachteten Seite konnte keine &Uuml;bereinstimmung mit dem definierten Suchmuster dieses Beobachtungskanals gefunden werden!";
					this.ResultStateLabel.Style["COLOR"] = "#ff0000";
					this.ResultTextLabel.Text = "";
					break;
				case UserDefinedMessageObservation.ResultStateEnum.MessageTemplateParamNotFound:
					this.ResultStateLabel.Text = "Es konnte keine passende Abbildung des aus dem Suchmuster resultierenden Ergebnisses auf die Meldungsvorlage gefunden werden!";
					this.ResultStateLabel.Style["COLOR"] = "#ff0000";
					this.ResultTextLabel.Text = "";
					break;
				case UserDefinedMessageObservation.ResultStateEnum.Error:
					this.ResultStateLabel.Text = "W&auml;hrend der &Uuml;berpr&uuml;fung ist ein Fehler aufgetreten - die Seite konnte nicht bezogen werden!";
					this.ResultStateLabel.Style["COLOR"] = "#ff0000";
					this.ResultTextLabel.Text = "";
					break;
			}
		} // end: initDetailView

		private void initGeneralPropertiesView(UserDefinedMessageObservation udmObs) 
		{
			this.NameLabel.Text = udmObs.Name;
			this.UrlLabel.Text = udmObs.Url.ToString();
			if(udmObs.ChangeState == ChangeStateEnum.Init) 
			{
				this.DateLabel.Text = "&Uuml;ber diesen Kanal wurde noch keine Beobachtung angesto&szlig;en.";
				this.ChangeStateLabel.Text = "Initialisiert";
			} 
			else 
			{
				this.DateLabel.Text = udmObs.Date.ToShortDateString() + "  " + udmObs.Date.ToShortTimeString();
				if(udmObs.ChangeState == ChangeStateEnum.Changed)
					this.ChangeStateLabel.Text = "Ver&auml;ndert";
				else if(udmObs.ChangeState == ChangeStateEnum.Unchanged) 
					this.ChangeStateLabel.Text = "Unver&auml;ndert";
				else if(udmObs.ChangeState == ChangeStateEnum.Error)
					this.ChangeStateLabel.Text = "&Uuml;berpr&uuml;fungsfehler aufgetreten";
				else if(udmObs.ChangeState == ChangeStateEnum.New)
					this.ChangeStateLabel.Text = "Neu";
				else
					this.ChangeStateLabel.Text = "-";
			}
		} // end: initGeneralPropertiesView
		
		protected void CheckButtonClicked(object sender, EventArgs args) 
		{
			int id = (int)Session["CurrentUserDefinedMessageObservationDetailId"];
			UserDefinedMessageObservation udmObs;
			string errorMsg;
			UserDefinedMessageObservationMediator.ReadResult res = UserDefinedMessageObservationMediator.LoadUserDefinedMessageObservation(
				id,
				out udmObs,
				out errorMsg);
			if(res == UserDefinedMessageObservationMediator.ReadResult.Ok) 
			{
				int newCheckCounterValue = ((int)Session["UserCheckCounter"]) + 1;
				CheckResult checkRes = udmObs.Check(newCheckCounterValue, out errorMsg);
				if(checkRes == CheckResult.Ok) 
				{
					// Update users CheckCounter in database 
					UserDataWrapper udw = new UserDataWrapper(User.Identity.Name);
					UsersMediator.UpdateResult ccUpdRes = UsersMediator.UpdateUsersCheckCounter(
						udw.Userid, 
						newCheckCounterValue, 
						out errorMsg);
					if(ccUpdRes != UsersMediator.UpdateResult.Ok) 
					{
						Session["ErrorMessage"] = "Beim Aktualisieren der Datenbank ist ein Problem aufgetreten.";
						Response.Redirect("./ErrorPage.aspx");
						return;
					}
					// Update Users CheckCounter in Session
					Session["UserCheckCounter"] = newCheckCounterValue;
					// Update checked UserDefinedMessageObservation in database
					UserDefinedMessageObservationMediator.UpdateResult updRes = UserDefinedMessageObservationMediator.UpdateCheckedUserDefinedMessageObservation(udmObs, out errorMsg);
					if(updRes != UserDefinedMessageObservationMediator.UpdateResult.Ok) 
					{
						Session["ErrorMessage"] = "Beim Aktualisieren der Datenbank ist ein Problem aufgetreten.";
						Response.Redirect("./ErrorPage.aspx");
						return;
					}
					else 
					{
						// Update UserDefinedMessageObservation view
						this.initGeneralPropertiesView(udmObs);
						this.initDetailView(udmObs);
					}
				} 
				else 
				{
					Session["ErrorMessage"] = "Beim Beobachten ist ein Problem aufgetreten.";
					Response.Redirect("./ErrorPage.aspx");
					return;
				}
			} 
			else 
			{
				Session["ErrorMessage"] = "Beim Lesen des Beobachtungskanals aus der Datenbank ist ein Problem aufgetreten.";
				Response.Redirect("./ErrorPage.aspx");
				return;
			}
		} // end: method CheckButtonClicked

		protected void SettingsButtonClicked(object sender, EventArgs args) 
		{
			Response.Redirect("./UserDefinedMessageObservationEditProperties.aspx");
		}

		protected void DeleteButtonClicked(object sender, EventArgs args) 
		{
			int id = (int)Session["CurrentUserDefinedMessageObservationDetailId"];
			string errorMsg;
			ObservationsMediator.DeleteResult res = ObservationsMediator.DeleteObservation(id, out errorMsg);
			if(res == ObservationsMediator.DeleteResult.Ok) 
			{
				Session.Remove("CurrentUserDefinedMessageObservationDetailId");
				Response.Redirect("./CategoryContent.aspx");
			} 
			else if(res == ObservationsMediator.DeleteResult.InvalidParams) 
			{
				Session["ErrorMessage"] = "Es konnte kein passender zu l&ouml;schender Beobachtungskanal gefunden werden.";
				Response.Redirect("./ErrorPage.aspx");
			} 
			else if(res == ObservationsMediator.DeleteResult.Error)  
			{
				Session["ErrorMessage"] = "Beim L&ouml;schen aus der Datenbank ist ein Problem aufgetreten.";
				Response.Redirect("./ErrorPage.aspx");
			}
		} // end: method DeleteButtonClicked


		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Load += new System.EventHandler(this.Page_Load);
		}
		#endregion
	}
}
