using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using WebseitenMonitor.DomainObjects.Behaviour;
using WebseitenMonitor.DataAccess;
using WebseitenMonitor.Ui.CustomControls;
using WebseitenMonitor.User.Help;
using log4net;

namespace WebseitenMonitor.User
{
	/// <summary>
	/// Summary description for UserDefinedMessageObservationEditProperties.
	/// </summary>
	public class UserDefinedMessageObservationEditProperties : System.Web.UI.Page
	{
		private static readonly ILog log = LogManager.GetLogger(typeof(UserDefinedMessageObservationEditProperties));

		protected System.Web.UI.WebControls.TextBox NameTextBox;
		protected System.Web.UI.WebControls.RequiredFieldValidator NameRequiredFieldValidator;
		protected System.Web.UI.WebControls.CustomValidator NameCustomValidator;
		protected System.Web.UI.WebControls.TextBox UrlTextBox;
		protected System.Web.UI.WebControls.RequiredFieldValidator UrlRequiredFieldValidator;
		protected System.Web.UI.WebControls.CustomValidator UrlCustomValidator;
		protected System.Web.UI.WebControls.CheckBox SearchPatternEnabledCheckBox;
		protected System.Web.UI.WebControls.TextBox SearchPatternTextBox;
		protected System.Web.UI.WebControls.CustomValidator SearchPatternCustomValidator;
		protected System.Web.UI.WebControls.TextBox MessageTemplateTextBox;
		protected System.Web.UI.WebControls.CustomValidator MessageTemplateCustomValidator;
		protected System.Web.UI.WebControls.TextBox SelectedElementsPositionDescriptorTextBox;
		protected System.Web.UI.WebControls.CustomValidator SelectedElementsPositionDescriptorCustomValidator;
		protected System.Web.UI.WebControls.Button OkButton;
		protected System.Web.UI.WebControls.RequiredFieldValidator SearchPatternRequiredFieldValidator;
		protected System.Web.UI.WebControls.RequiredFieldValidator MessageTemplateRequiredFieldValidator;
		protected System.Web.UI.WebControls.RequiredFieldValidator SelectedElementsPositionDescriptorRequiredFieldValidator;
		protected System.Web.UI.WebControls.Button CancelButton;
		protected Fold NameHelpFold;
		protected Fold UrlHelpFold;
		protected Fold SearchPatternEnableHelpFold;
		protected Fold SearchPatternHelpFold;
		protected Fold MessageTemplateHelpFold;
		protected System.Web.UI.WebControls.Button PreviewButton;
		protected System.Web.UI.WebControls.Label PreviewHeaderLabel;
		protected System.Web.UI.WebControls.RadioButton HtmlPreviewRadioButton;
		protected System.Web.UI.WebControls.RadioButton TextPreviewRadioButton;
		protected System.Web.UI.WebControls.Label PreviewErrorLabel;
		protected System.Web.UI.WebControls.Label PreviewHtmlLabel;
		protected System.Web.UI.WebControls.TextBox PreviewTextTextBox;
		protected Fold SelectedElementsPositionDescriptorHelpFold;
	
		private void Page_Load(object sender, System.EventArgs e)
		{
			// Show current Properties as default values
			if(!IsPostBack) 
			{
				int id = (int)Session["CurrentUserDefinedMessageObservationDetailId"];
				UserDefinedMessageObservation udmObs;
				string errorMsg;
				UserDefinedMessageObservationMediator.ReadResult res = UserDefinedMessageObservationMediator.LoadUserDefinedMessageObservation(
					id,
					out udmObs,
					out errorMsg);
				if(res == UserDefinedMessageObservationMediator.ReadResult.Ok) 
				{
					this.NameTextBox.Text = udmObs.Name;
					this.UrlTextBox.Text = udmObs.Url.ToString();
					if(udmObs.SearchPatternEnabled) 
					{
						this.SearchPatternEnabledCheckBox.Checked = true;
						this.SearchPatternTextBox.Text = udmObs.SearchPattern;
						this.MessageTemplateTextBox.Text = udmObs.MessageTemplate;
						this.SearchPatternTextBox.Enabled = true;
						this.MessageTemplateTextBox.Enabled = true;
						this.SearchPatternRequiredFieldValidator.Enabled = true;
						this.MessageTemplateRequiredFieldValidator.Enabled = true;
						this.SearchPatternCustomValidator.Enabled = true;
						this.MessageTemplateCustomValidator.Enabled = true;
						this.SearchPatternHelpFold.Visible = true;
						this.MessageTemplateHelpFold.Visible = true;
					} 
					else 
					{
						this.SearchPatternEnabledCheckBox.Checked = false;
						this.SearchPatternTextBox.Text = "";
						this.MessageTemplateTextBox.Text = "";
						this.SearchPatternTextBox.Enabled = false;
						this.MessageTemplateTextBox.Enabled = false;
						this.SearchPatternRequiredFieldValidator.Enabled = false;
						this.MessageTemplateRequiredFieldValidator.Enabled = false;
						this.SearchPatternCustomValidator.Enabled = false;
						this.MessageTemplateCustomValidator.Enabled = false;
						this.SearchPatternHelpFold.Visible = false;
						this.MessageTemplateHelpFold.Visible = false;
					}
					this.SelectedElementsPositionDescriptorTextBox.Text = udmObs.SelectedElementsPositionDescriptor;
					this.NameHelpFold.AlternateText = HelpTextConstants.NameHelpHtmlText;
					this.UrlHelpFold.AlternateText = HelpTextConstants.UrlHelpHtmlText;
					this.SearchPatternEnableHelpFold.AlternateText = HelpTextConstants.SearchPatternEnableHelpHtmlText;
					this.SearchPatternHelpFold.AlternateText = HelpTextConstants.SearchPatternHelpHtmlText;
					this.MessageTemplateHelpFold.AlternateText = HelpTextConstants.MessageTemplateHelpHtmlText;
					this.SelectedElementsPositionDescriptorHelpFold.AlternateText = HelpTextConstants.SelectedElementsPositionDescriptorHelpHtmlText;
					// Don't set Preview specific elements visible by default
					this.PreviewHeaderLabel.Visible = false;
					this.HtmlPreviewRadioButton.Visible = false;
					this.TextPreviewRadioButton.Visible = false;
					this.HtmlPreviewRadioButton.Checked = true;
					this.TextPreviewRadioButton.Checked = false;
					this.PreviewErrorLabel.Visible = false;
					this.PreviewHtmlLabel.Visible = false;
					this.PreviewTextTextBox.Visible = false;
				} 
				else 
				{
					Session["ErrorMessage"] = "Beim Lesen des Beobachtungskanals aus der Datenbank ist ein Problem aufgetreten.";
					Response.Redirect("./ErrorPage.aspx");
					return;
				}
			}
		}

		protected void ValidateName(object source, ServerValidateEventArgs args) 
		{
			// Name of ChangeObservation must at least be one sign long
			if(args.Value == null || args.Value.Length <= 0) 
			{
				((CustomValidator)source).ErrorMessage = "Es muss ein Name f&uuml;r den Beobachtungskanal eingegeben werden!";
				args.IsValid = false;
			}
				// Name must not have more than 50 sign's - DB constraint
			else if(args.Value.Length > 50) 
			{
				((CustomValidator)source).ErrorMessage = "Der Name des Beobachtungskanals darf maximal 50 Zeichen lang sein!";
				args.IsValid = false;
			}
			else 
			{
				// only Letters, Digits, - and _ are allowed
				foreach(char tmpChar in args.Value) 
				{
					if(!Char.IsLetterOrDigit(tmpChar) && !(tmpChar == '-') && !(tmpChar == '_')) 
					{
						((CustomValidator)source).ErrorMessage = "Der Name des Beobachtungskanals darf nur aus Buchstaben, Ziffern, &acute;-&acute; und &acute;_&acute; bestehen!";
						args.IsValid = false;
						break;
					}
				}
			}
		}

		protected void ValidateUrl(object source, ServerValidateEventArgs args) 
		{
			// Url of ChangeObservation must at least be one sign long
			if(args.Value == null || args.Value.Length <= 0) 
			{
				((CustomValidator)source).ErrorMessage = "Es muss eine der Beobachtung zugrunde liegende Url angegeben werden!";
				args.IsValid = false;
			} 
			else
			{
				try 
				{
					Uri tmpUri = new Uri(args.Value);
				} 
				catch 
				{
					((CustomValidator)source).ErrorMessage = "Die eingegebene Url ist syntaktisch ung&uuml;ltig!";
					args.IsValid = false;
				}
			}
		}

		protected void SearchPatternEnableChanged(object sender, EventArgs args) 
		{
			if(this.SearchPatternEnabledCheckBox.Checked) 
			{
				this.SearchPatternTextBox.Enabled = true;
				this.MessageTemplateTextBox.Enabled = true;
				this.SearchPatternRequiredFieldValidator.Enabled = true;
				this.MessageTemplateRequiredFieldValidator.Enabled = true;
				this.SearchPatternCustomValidator.Enabled = true;
				this.MessageTemplateCustomValidator.Enabled = true;
				this.SearchPatternHelpFold.Visible = true;
				this.MessageTemplateHelpFold.Visible = true;
			} 
			else 
			{
				this.SearchPatternTextBox.Enabled = false;
				this.MessageTemplateTextBox.Enabled = false;
				this.SearchPatternRequiredFieldValidator.Enabled = false;
				this.MessageTemplateRequiredFieldValidator.Enabled = false;
				this.SearchPatternCustomValidator.Enabled = false;
				this.MessageTemplateCustomValidator.Enabled = false;
				this.SearchPatternHelpFold.Visible = false;
				this.MessageTemplateHelpFold.Visible = false;
			}
		}

		protected void ValidateSearchPattern(object source, ServerValidateEventArgs args) 
		{
			string errorMsg;
			if(!UserDefinedMessageObservation.CheckSearchPattern(args.Value, out errorMsg)) 
			{
				((CustomValidator)source).ErrorMessage = errorMsg;
				args.IsValid = false;
			}
		}

		protected void ValidateMessageTemplate(object source, ServerValidateEventArgs args) 
		{
			string errorMsg;
			if(!UserDefinedMessageObservation.CheckMessageTemplate(args.Value, out errorMsg)) 
			{
				((CustomValidator)source).ErrorMessage = errorMsg;
				args.IsValid = false;
			}
		}

		protected void ValidateSelectedElementsPositionDescriptor(object source, ServerValidateEventArgs args) 
		{
			string errorMsg;
			if(!UserDefinedMessageObservation.CheckSelectedElementsPositionDescriptor(args.Value, out errorMsg)) 
			{
				((CustomValidator)source).ErrorMessage = errorMsg;
				args.IsValid = false;
			}
		}

		protected void OkButtonClicked(object sender, EventArgs args) 
		{
			if(Page.IsValid) 
			{
				int id = (int)Session["CurrentUserDefinedMessageObservationDetailId"];
				UserDefinedMessageObservation udmObs;
				string errorMsg;
				UserDefinedMessageObservationMediator.ReadResult res = UserDefinedMessageObservationMediator.LoadUserDefinedMessageObservation(
					id,
					out udmObs,
					out errorMsg);
				if(res != UserDefinedMessageObservationMediator.ReadResult.Ok) 
				{	
					Session["ErrorMessage"] = "Beim Laden des Beobachtungskanals aus der Datenbank ist ein Problem aufgetreten!";
					Response.Redirect("./ErrorPage.aspx");
					return;
				}
				// Change Properties in object-representation
				UserDefinedMessageObservation.ChangePropertiesEnum changeRes = udmObs.ChangeProperties(
					this.NameTextBox.Text, 
					new Uri(this.UrlTextBox.Text), 
					this.SearchPatternEnabledCheckBox.Checked, 
					(this.SearchPatternEnabledCheckBox.Checked)?this.SearchPatternTextBox.Text:null, 
					(this.SearchPatternEnabledCheckBox.Checked)?this.MessageTemplateTextBox.Text:null,
					this.SelectedElementsPositionDescriptorTextBox.Text);
				if(changeRes != UserDefinedMessageObservation.ChangePropertiesEnum.NoChange) 
				{
					if(changeRes == UserDefinedMessageObservation.ChangePropertiesEnum.Inconsistence) 
					{
						Session["ErrorMessage"] = "Beim Aktualisieren der Eigenschaften des Beobachtungskanals ist ein Problem aufgetreten!";
						Response.Redirect("./ErrorPage.aspx");
						return;
					} 
					else if(changeRes == UserDefinedMessageObservation.ChangePropertiesEnum.Change) // something changed
					{
						// Update Changes to database
						UserDefinedMessageObservationMediator.UpdateResult updateRes = UserDefinedMessageObservationMediator.UpdateChangedPropertiesUserDefinedMessageObservation(udmObs, out errorMsg);
						if(updateRes != UserDefinedMessageObservationMediator.UpdateResult.Ok) 
						{
							Session["ErrorMessage"] = "Beim Aktualisieren der Eigenschaften des Beobachtungskanals in der Datenbank ist ein Problem aufgetreten!";
							Response.Redirect("./ErrorPage.aspx");
							return;
						}
					}
				}
				Response.Redirect("./UserDefinedMessageObservationDetail.aspx");
			}
		}

		protected void CancelButtonClicked(object sender, EventArgs args) 
		{
			Response.Redirect("./UserDefinedMessageObservationDetail.aspx");
		}

		protected void PreviewButtonClicked(object sender, EventArgs args) 
		{
			if(this.Page.IsValid) 
			{
				string previewText;
				string errorMsg;
				UserDefinedMessageObservation.PreviewResult prevRes = UserDefinedMessageObservation.GetPreview(
					this.UrlTextBox.Text,
					this.SearchPatternEnabledCheckBox.Checked,
					this.SearchPatternTextBox.Text,
					this.MessageTemplateTextBox.Text,
					this.SelectedElementsPositionDescriptorTextBox.Text,
					out previewText,
					out errorMsg);
				this.PreviewErrorLabel.Text = "";
				this.PreviewHtmlLabel.Text = "";
				this.PreviewTextTextBox.Text = "";
				switch(prevRes) 
				{
					case UserDefinedMessageObservation.PreviewResult.Ok:
						this.PreviewHtmlLabel.Text = previewText;
						this.PreviewTextTextBox.Text = previewText;
						break;
					case UserDefinedMessageObservation.PreviewResult.InvalidParams:
						this.PreviewErrorLabel.Text = "Parameter sind ung&uuml;tig!";
						break;
					case UserDefinedMessageObservation.PreviewResult.RetrieveError:
						this.PreviewErrorLabel.Text = "Die angegebene Seite konnte nicht bezogen werden oder ist keine Html-Seite.";
						break;
					case UserDefinedMessageObservation.PreviewResult.NoSelectionMatch:
						this.PreviewErrorLabel.Text = "Nach der angegebenen Selektionsbeschreibung konnte keine &Uuml;bereinstimmung erzielt werden.";
						break;
					case UserDefinedMessageObservation.PreviewResult.NoRegExMatch:
						this.PreviewErrorLabel.Text = "Nach dem angegenbenen Suchmuster konnte kein Treffer gefunden werden.";
						break;
					case UserDefinedMessageObservation.PreviewResult.RegExParamNotFound:
						this.PreviewErrorLabel.Text = "Es konnten nicht alle Parameter der Meldungsvorlage aufgel&ouml;st werden:<br>" + errorMsg;
						break;
					default:
						this.PreviewErrorLabel.Text = "Es ist ein undefinierter Fehler aufgetreten.";
						break;
				}
				// set visibility
				this.PreviewHeaderLabel.Visible = true;
				this.HtmlPreviewRadioButton.Visible = true;
				this.TextPreviewRadioButton.Visible = true;
				if(prevRes == UserDefinedMessageObservation.PreviewResult.Ok) 
				{
					this.PreviewErrorLabel.Visible = false;
					if(this.HtmlPreviewRadioButton.Checked) 
					{
						this.PreviewHtmlLabel.Visible = true;
						this.PreviewTextTextBox.Visible = false;
					} 
					else 
					{
						this.PreviewTextTextBox.Visible = true;
						this.PreviewHtmlLabel.Visible = false;
					}
				} 
				else 
				{
					this.PreviewErrorLabel.Visible = true;
					this.PreviewHtmlLabel.Visible = false;
					this.PreviewTextTextBox.Visible = false;
				}
			} 
			else // !this.Page.IsValid
			{
				this.PreviewHeaderLabel.Visible = false;
				this.HtmlPreviewRadioButton.Visible = false;
				this.TextPreviewRadioButton.Visible = false;
				this.PreviewErrorLabel.Visible = false;
				this.PreviewErrorLabel.Text = "";
				this.PreviewHtmlLabel.Visible = false;
				this.PreviewHtmlLabel.Text = "";
				this.PreviewTextTextBox.Visible = false;
				this.PreviewTextTextBox.Text = "";
			}
		} // end: method PreviewButtonClicked

		protected void PreviewModeChanged(object sender, EventArgs args) 
		{
			if(this.HtmlPreviewRadioButton.Checked) 
			{
				this.PreviewHtmlLabel.Visible = true;
				this.PreviewTextTextBox.Visible = false;
			} 
			else 
			{
				this.PreviewTextTextBox.Visible = true;
				this.PreviewHtmlLabel.Visible = false;
			}
		} // end: method PreviewModeChanged

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion
	}
}
